# -*- coding: utf-8 -*-
#
# Copyright © 2009-2010 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""Simple application based on guiqwt and guidata"""

SHOW = True  # Show test in GUI-based test launcher

from qtpy.QtWidgets import QMainWindow, QMessageBox, QSplitter, QListWidget
from qtpy.QtCore import QSize, __version__, Qt
from qtpy.compat import getopenfilename

import sys, platform
import numpy as np

from guidata.dataset.datatypes import DataSet, GetAttrProp
from guidata.dataset.dataitems import IntItem, FloatArrayItem, StringItem, ChoiceItem
from guidata.dataset.qtwidgets import DataSetEditGroupBox
from guidata.configtools import get_icon
from guidata.qthelpers import create_action, add_actions, get_std_icon
from guidata.utils import update_dataset
from qtpy.py3compat import to_text_string

from guiqwt.config import _
from guiqwt.plot import ImageWidget
from guiqwt.builder import make
from guiqwt import io

APP_NAME = _("Application example")
VERSION = "1.0.0"


class ImageParam(DataSet):
    _hide_data = False
    _hide_size = True
    title = StringItem(_("Title"), default=_("Untitled"))
    data = FloatArrayItem(_("Data")).set_prop("display", hide=GetAttrProp("_hide_data"))
    width = IntItem(
        _("Width"), help=_("Image width (pixels)"), min=1, default=100
    ).set_prop("display", hide=GetAttrProp("_hide_size"))
    height = IntItem(
        _("Height"), help=_("Image height (pixels)"), min=1, default=100
    ).set_prop("display", hide=GetAttrProp("_hide_size"))


class ImageParamNew(ImageParam):
    _hide_data = True
    _hide_size = False
    type = ChoiceItem(_("Type"), (("rand", _("random")), ("zeros", _("zeros"))))


class ImageListWithProperties(QSplitter):
    def __init__(self, parent):
        QSplitter.__init__(self, parent)
        self.imagelist = QListWidget(self)
        self.addWidget(self.imagelist)
        self.properties = DataSetEditGroupBox(_("Properties"), ImageParam)
        self.properties.setEnabled(False)
        self.addWidget(self.properties)


class CentralWidget(QSplitter):
    def __init__(self, parent, toolbar):
        QSplitter.__init__(self, parent)
        self.setContentsMargins(10, 10, 10, 10)
        self.setOrientation(Qt.Vertical)

        imagelistwithproperties = ImageListWithProperties(self)
        self.addWidget(imagelistwithproperties)
        self.imagelist = imagelistwithproperties.imagelist
        self.imagelist.currentRowChanged.connect(self.current_item_changed)
        self.imagelist.itemSelectionChanged.connect(self.selection_changed)
        self.properties = imagelistwithproperties.properties
        self.properties.SIG_APPLY_BUTTON_CLICKED.connect(self.properties_changed)

        self.imagewidget = ImageWidget(self)
        self.imagewidget.plot.SIG_LUT_CHANGED.connect(self.lut_range_changed)
        self.item = None  # image item

        self.imagewidget.add_toolbar(toolbar, "default")
        self.imagewidget.register_all_image_tools()

        self.addWidget(self.imagewidget)

        self.images = []  # List of ImageParam instances
        self.lut_ranges = []  # List of LUT ranges

        self.setStretchFactor(0, 0)
        self.setStretchFactor(1, 1)
        self.setHandleWidth(10)
        self.setSizes([1, 2])

    def refresh_list(self):
        self.imagelist.clear()
        self.imagelist.addItems([image.title for image in self.images])

    def selection_changed(self):
        """Image list: selection changed"""
        row = self.imagelist.currentRow()
        self.properties.setDisabled(row == -1)

    def current_item_changed(self, row):
        """Image list: current image changed"""
        image, lut_range = self.images[row], self.lut_ranges[row]
        self.show_data(image.data, lut_range)
        update_dataset(self.properties.dataset, image)
        self.properties.get()

    def lut_range_changed(self):
        row = self.imagelist.currentRow()
        self.lut_ranges[row] = self.item.get_lut_range()

    def show_data(self, data, lut_range=None):
        plot = self.imagewidget.plot
        if self.item is not None:
            self.item.set_data(data)
            if lut_range is None:
                lut_range = self.item.get_lut_range()
            self.imagewidget.set_contrast_range(*lut_range)
            self.imagewidget.update_cross_sections()
        else:
            self.item = make.image(data)
            plot.add_item(self.item, z=0)
        plot.replot()

    def properties_changed(self):
        """The properties 'Apply' button was clicked: updating image"""
        row = self.imagelist.currentRow()
        image = self.images[row]
        update_dataset(image, self.properties.dataset)
        self.refresh_list()
        self.show_data(image.data)

    def add_image(self, image):
        self.images.append(image)
        self.lut_ranges.append(None)
        self.refresh_list()
        self.imagelist.setCurrentRow(len(self.images) - 1)
        plot = self.imagewidget.plot
        plot.do_autoscale()

    def add_image_from_file(self, filename):
        image = ImageParam()
        image.title = to_text_string(filename)
        image.data = io.imread(filename, to_grayscale=True)
        image.height, image.width = image.data.shape
        self.add_image(image)


class MainWindow(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)
        self.setup()

    def setup(self):
        """Setup window parameters"""
        self.setWindowIcon(get_icon("python.png"))
        self.setWindowTitle(APP_NAME)
        self.resize(QSize(600, 800))

        # Welcome message in statusbar:
        status = self.statusBar()
        status.showMessage(_("Welcome to guiqwt application example!"), 5000)

        # File menu
        file_menu = self.menuBar().addMenu(_("File"))
        new_action = create_action(
            self,
            _("New..."),
            shortcut="Ctrl+N",
            icon=get_icon("filenew.png"),
            tip=_("Create a new image"),
            triggered=self.new_image,
        )
        open_action = create_action(
            self,
            _("Open..."),
            shortcut="Ctrl+O",
            icon=get_icon("fileopen.png"),
            tip=_("Open an image"),
            triggered=self.open_image,
        )
        quit_action = create_action(
            self,
            _("Quit"),
            shortcut="Ctrl+Q",
            icon=get_std_icon("DialogCloseButton"),
            tip=_("Quit application"),
            triggered=self.close,
        )
        add_actions(file_menu, (new_action, open_action, None, quit_action))

        # Help menu
        help_menu = self.menuBar().addMenu("?")
        about_action = create_action(
            self,
            _("About..."),
            icon=get_std_icon("MessageBoxInformation"),
            triggered=self.about,
        )
        add_actions(help_menu, (about_action,))

        main_toolbar = self.addToolBar("Main")
        add_actions(main_toolbar, (new_action, open_action,))

        # Set central widget:
        toolbar = self.addToolBar("Image")
        self.mainwidget = CentralWidget(self, toolbar)
        self.setCentralWidget(self.mainwidget)

    # ------?
    def about(self):
        QMessageBox.about(
            self,
            _("About ") + APP_NAME,
            """<b>%s</b> v%s<p>%s Pierre Raybaut
              <br>Copyright &copy; 2009-2010 CEA
              <p>Python %s, Qt %s %s %s"""
            % (
                APP_NAME,
                VERSION,
                _("Developped by"),
                platform.python_version(),
                __version__,
                _("on"),
                platform.system(),
            ),
        )

    # ------I/O
    def new_image(self):
        """Create a new image"""
        imagenew = ImageParamNew(title=_("Create a new image"))
        if not imagenew.edit(self):
            return
        image = ImageParam()
        image.title = imagenew.title
        if imagenew.type == "zeros":
            image.data = np.zeros((imagenew.width, imagenew.height))
        elif imagenew.type == "rand":
            image.data = np.random.randn(imagenew.width, imagenew.height)
        self.mainwidget.add_image(image)

    def open_image(self):
        """Open image file"""
        saved_in, saved_out, saved_err = sys.stdin, sys.stdout, sys.stderr
        sys.stdout = None
        filename, _filter = getopenfilename(
            self, _("Open"), "", io.iohandler.get_filters("load")
        )
        sys.stdin, sys.stdout, sys.stderr = saved_in, saved_out, saved_err
        if filename:
            self.mainwidget.add_image_from_file(filename)


if __name__ == "__main__":
    from guidata import qapplication

    app = qapplication()
    window = MainWindow()
    window.show()
    app.exec_()



# -*- coding: utf-8 -*-
#
# Copyright © 2009-2011 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""Oblique averaged cross section test"""

SHOW = True # Show test in GUI-based test launcher

import os.path as osp

import guiqwt.cross_section
# debug mode shows the ROI in the top-left corner of the image plot:
guiqwt.cross_section.DEBUG = True

from guiqwt.plot import ImageDialog
from guiqwt.builder import make
from guiqwt.tools import ImageMaskTool
from guiqwt.cross_section import ObliqueCrossSection
from guiqwt.tools import ObliqueCrossSectionTool, OCSPanelTool


class OCSImageDialog(ImageDialog):
    def register_image_tools(self):
        ImageDialog.register_image_tools(self)
        for tool in (ObliqueCrossSectionTool, OCSPanelTool, ImageMaskTool):
            self.add_tool(tool)
        
    def create_plot(self, options, row=0, column=0, rowspan=1, columnspan=1):
        ImageDialog.create_plot(self, options, row, column, rowspan, columnspan)
        ra_panel = ObliqueCrossSection(self)
        splitter = self.plot_widget.xcsw_splitter
        splitter.addWidget(ra_panel)
        splitter.setStretchFactor(splitter.count()-1, 1)
        splitter.setSizes(list(splitter.sizes())+[2])
        self.add_panel(ra_panel)

def test():
    """Test"""
    # -- Create QApplication
    import guidata
    _app = guidata.qapplication()
    # --
    win = OCSImageDialog(toolbar=True,
                         wintitle="Oblique averaged cross section test")
    win.resize(600, 600)
    
#    from guiqwt.tests.image import compute_image
#    data = np.array((compute_image(4000, grid=False)+1)*32e3, dtype=np.uint16)
#    image = make.maskedimage(data, colormap="bone", show_mask=True)

    filename = osp.join(osp.dirname(__file__), "brain_cylinder.png")
    image = make.maskedimage(filename=filename, colormap="bone")

    plot = win.get_plot()
    plot.add_item(image)
    win.exec_()

if __name__ == "__main__":
    test()



# -*- coding: utf-8 -*-
"""
Basic geometry functions
"""

# pylint: disable=C0103

from numpy import (matrix, array, arccos, sign, cos, sin, linalg, vdot,
                   pi, sqrt, fabs, arctan)


#===============================================================================
# Transform matrix functions
#===============================================================================

def translate(tx, ty):
    """Return translation matrix (NumPy matrix object)"""
    return matrix([[1, 0, tx],
                   [0, 1, ty],
                   [0, 0, 1 ]], float)

def scale(sx, sy):
    """Return scale matrix (NumPy matrix object)"""
    return matrix([[sx, 0,  0],
                   [0,  sy, 0],
                   [0,  0,  1]], float)

def rotate(alpha):
    """Return rotation matrix (NumPy matrix object)"""
    return matrix([[cos(alpha), -sin(alpha), 0],
                   [sin(alpha),  cos(alpha), 0],
                   [0,           0,          1]], float)

def colvector(x, y):
    """Return vector (NumPy matrix object) from coordinates"""
    return matrix([x, y, 1]).T


#===============================================================================
# Operations on vectors (from coordinates)
#===============================================================================

def vector_norm(xa, ya, xb, yb):
    """Return vector norm: (xa, xb)-->(ya, yb)"""
    return linalg.norm(array((xb-xa, yb-ya)))

def vector_projection(dv, xa, ya, xb, yb):
    """Return vector projection on *dv*: (xa, xb)-->(ya, yb)"""
    assert dv.shape == (2,)
    v_ab = array((xb-xa, yb-ya))
    u_ab = v_ab/linalg.norm(v_ab)
    return vdot(u_ab, dv)*u_ab+array((xb, yb))

def vector_rotation(theta, dx, dy):
    """Compute theta-rotation on vector *v*, returns vector coordinates"""
    return array( rotate(theta)*colvector(dx, dy) ).ravel()[:2]

def vector_angle(dx, dy):
    """Return vector angle with X-axis"""
    # sign(dy) ==  1 --> return Arccos()
    # sign(dy) ==  0 --> return  0 if sign(dx) ==  1
    # sign(dy) ==  0 --> return pi if sign(dx) == -1
    # sign(dy) == -1 --> return 2pi-Arccos()
    if dx == 0 and dy == 0:
        return 0.
    else:
        sx, sy = sign(dx), sign(dy)
        acos = arccos(dx/sqrt(dx**2+dy**2))
        return sy*(pi*(sy-1)+acos)+pi*(1-sy**2)*(1-sx)*.5


#===============================================================================
# Misc.
#===============================================================================

def compute_center(x1, y1, x2, y2):
    return .5*(x1+x2), .5*(y1+y2)
    
def compute_rect_size(x1, y1, x2, y2):
    return x2-x1, fabs(y2-y1)

def compute_distance(x1, y1, x2, y2):
    return sqrt((x2-x1)**2+(y2-y1)**2)
    
def compute_angle(x1, y1, x2, y2, reverse=False):
    sign = -1 if reverse else 1
    if x2 == x1:
        return 0.
    else:
        return arctan(-sign*(y2-y1)/(x2-x1))*180/pi



# -*- coding: utf-8 -*-
#
# Copyright © 2011 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guidata/__init__.py for details)

"""
qthelpers
---------

The ``guiqwt.qthelpers`` module provides helper functions for developing 
easily Qt-based graphical user interfaces with guiqwt.

Ready-to-use open/save dialogs:
    :py:data:`guiqwt.qthelpers.exec_image_save_dialog`
        Executes an image save dialog box (QFileDialog.getSaveFileName)
    :py:data:`guiqwt.qthelpers.exec_image_open_dialog`
        Executes an image open dialog box (QFileDialog.getOpenFileName)
    :py:data:`guiqwt.qthelpers.exec_images_open_dialog`
        Executes an image*s* open dialog box (QFileDialog.getOpenFileNames)   

Reference
~~~~~~~~~

.. autofunction:: exec_image_save_dialog
.. autofunction:: exec_image_open_dialog
.. autofunction:: exec_images_open_dialog
"""

import sys
import os.path as osp

from qtpy.QtWidgets import QMessageBox
from qtpy.compat import getsavefilename, getopenfilename, getopenfilenames

from qtpy.py3compat import to_text_string

# Local imports
from guiqwt.config import _
from guiqwt import io


# ===============================================================================
# Ready-to-use open/save dialogs
# ===============================================================================
def exec_image_save_dialog(parent, data, template=None, basedir="", app_name=None):
    """
    Executes an image save dialog box (QFileDialog.getSaveFileName)
        * parent: parent widget (None means no parent)
        * data: image pixel array data
        * template: image template (pydicom dataset) for DICOM files
        * basedir: base directory ('' means current directory)
        * app_name (opt.): application name (used as a title for an eventual 
          error message box in case something goes wrong when saving image)
    
    Returns filename if dialog is accepted, None otherwise
    """
    saved_in, saved_out, saved_err = sys.stdin, sys.stdout, sys.stderr
    sys.stdout = None
    filename, _filter = getsavefilename(
        parent,
        _("Save as"),
        basedir,
        io.iohandler.get_filters("save", dtype=data.dtype, template=template),
    )
    sys.stdin, sys.stdout, sys.stderr = saved_in, saved_out, saved_err
    if filename:
        filename = to_text_string(filename)
        kwargs = {}
        if osp.splitext(filename)[1].lower() == ".dcm":
            kwargs["template"] = template
        try:
            io.imwrite(filename, data, **kwargs)
            return filename
        except Exception as msg:
            import traceback

            traceback.print_exc()
            QMessageBox.critical(
                parent,
                _("Error") if app_name is None else app_name,
                (_("%s could not be written:") % osp.basename(filename))
                + "\n"
                + str(msg),
            )
            return


def exec_image_open_dialog(
    parent, basedir="", app_name=None, to_grayscale=True, dtype=None
):
    """
    Executes an image open dialog box (QFileDialog.getOpenFileName)
        * parent: parent widget (None means no parent)
        * basedir: base directory ('' means current directory)
        * app_name (opt.): application name (used as a title for an eventual 
          error message box in case something goes wrong when saving image)
        * to_grayscale (default=True): convert image to grayscale
    
    Returns (filename, data) tuple if dialog is accepted, None otherwise
    """
    saved_in, saved_out, saved_err = sys.stdin, sys.stdout, sys.stderr
    sys.stdout = None
    filename, _filter = getopenfilename(
        parent, _("Open"), basedir, io.iohandler.get_filters("load", dtype=dtype)
    )
    sys.stdin, sys.stdout, sys.stderr = saved_in, saved_out, saved_err
    filename = to_text_string(filename)
    try:
        data = io.imread(filename, to_grayscale=to_grayscale)
    except Exception as msg:
        import traceback

        traceback.print_exc()
        QMessageBox.critical(
            parent,
            _("Error") if app_name is None else app_name,
            (_("%s could not be opened:") % osp.basename(filename)) + "\n" + str(msg),
        )
        return
    return filename, data


def exec_images_open_dialog(
    parent, basedir="", app_name=None, to_grayscale=True, dtype=None
):
    """
    Executes an image*s* open dialog box (QFileDialog.getOpenFileNames)
        * parent: parent widget (None means no parent)
        * basedir: base directory ('' means current directory)
        * app_name (opt.): application name (used as a title for an eventual 
          error message box in case something goes wrong when saving image)
        * to_grayscale (default=True): convert image to grayscale
    
    Yields (filename, data) tuples if dialog is accepted, None otherwise
    """
    saved_in, saved_out, saved_err = sys.stdin, sys.stdout, sys.stderr
    sys.stdout = None
    filenames, _filter = getopenfilenames(
        parent, _("Open"), basedir, io.iohandler.get_filters("load", dtype=dtype)
    )
    sys.stdin, sys.stdout, sys.stderr = saved_in, saved_out, saved_err
    filenames = [to_text_string(fname) for fname in list(filenames)]
    for filename in filenames:
        try:
            data = io.imread(filename, to_grayscale=to_grayscale)
        except Exception as msg:
            import traceback

            traceback.print_exc()
            QMessageBox.critical(
                parent,
                _("Error") if app_name is None else app_name,
                (_("%s could not be opened:") % osp.basename(filename))
                + "\n"
                + str(msg),
            )
            return
        yield filename, data




# -*- coding: utf-8 -*-
#
# Copyright © 2009-2010 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

# pylint: disable=C0103

"""
guiqwt.annotations
------------------

The `annotations` module provides annotated shapes:
    * :py:class:`guiqwt.annotations.AnnotatedPoint`
    * :py:class:`guiqwt.annotations.AnnotatedSegment`
    * :py:class:`guiqwt.annotations.AnnotatedRectangle`
    * :py:class:`guiqwt.annotations.AnnotatedObliqueRectangle`
    * :py:class:`guiqwt.annotations.AnnotatedEllipse`
    * :py:class:`guiqwt.annotations.AnnotatedCircle`

An annotated shape is a plot item (derived from QwtPlotItem) that may be 
displayed on a 2D plotting widget like :py:class:`guiqwt.curve.CurvePlot` 
or :py:class:`guiqwt.image.ImagePlot`.

.. seealso:: module :py:mod:`guiqwt.shapes`

Examples
~~~~~~~~

An annotated shape may be created:
    * from the associated plot item class (e.g. `AnnotatedCircle` to 
      create an annotated circle): the item properties are then assigned 
      by creating the appropriate style parameters object
      (:py:class:`guiqwt.styles.AnnotationParam`)
      
>>> from guiqwt.annotations import AnnotatedCircle
>>> from guiqwt.styles import AnnotationParam
>>> param = AnnotationParam()
>>> param.title = 'My circle'
>>> circle_item = AnnotatedCircle(0., 2., 4., 0., param)
      
    * or using the `plot item builder` (see :py:func:`guiqwt.builder.make`):
      
>>> from guiqwt.builder import make
>>> circle_item = make.annotated_circle(0., 2., 4., 0., title='My circle')

Reference
~~~~~~~~~

.. autoclass:: AnnotatedPoint
   :members:
   :inherited-members:
.. autoclass:: AnnotatedSegment
   :members:
   :inherited-members:
.. autoclass:: AnnotatedRectangle
   :members:
   :inherited-members:
.. autoclass:: AnnotatedObliqueRectangle
   :members:
   :inherited-members:
.. autoclass:: AnnotatedEllipse
   :members:
   :inherited-members:
.. autoclass:: AnnotatedCircle
   :members:
   :inherited-members:
"""

from __future__ import unicode_literals

import numpy as np

from guidata.utils import update_dataset, assert_interfaces_valid

# Local imports
from guiqwt.config import CONF, _
from guiqwt.styles import LabelParam, AnnotationParam
from guiqwt.shapes import (AbstractShape, RectangleShape, EllipseShape,
                           SegmentShape, PointShape, ObliqueRectangleShape)
from guiqwt.label import DataInfoLabel
from guiqwt.interfaces import IBasePlotItem, IShapeItemType, ISerializableType
from guiqwt.geometry import (compute_center, compute_rect_size,
                             compute_distance, compute_angle)
from guiqwt.baseplot import canvas_to_axes


class AnnotatedShape(AbstractShape):
    """
    Construct an annotated shape with properties set with
    *annotationparam* (see :py:class:`guiqwt.styles.AnnotationParam`)
    """
    __implements__ = (IBasePlotItem, ISerializableType)
    SHAPE_CLASS = None
    LABEL_ANCHOR = None
    def __init__(self, annotationparam=None):
        AbstractShape.__init__(self)
        assert self.LABEL_ANCHOR is not None
        self.shape = self.create_shape()
        self.label = self.create_label()
        self.area_computations_visible = True
        if annotationparam is None:
            self.annotationparam = AnnotationParam(_("Annotation"),
                                                   icon="annotation.png")
        else:
            self.annotationparam = annotationparam
            self.annotationparam.update_annotation(self)
        
    def types(self):
        return (IShapeItemType, ISerializableType)
    
    def __reduce__(self):
        self.annotationparam.update_param(self)
        state = (self.shape, self.label, self.annotationparam)
        return (self.__class__, (), state)

    def __setstate__(self, state):
        shape, label, param = state
        self.shape = shape
        self.label = label
        self.annotationparam = param
        self.annotationparam.update_annotation(self)

    def serialize(self, writer):
        """Serialize object to HDF5 writer"""
        writer.write(self.annotationparam, group_name='annotationparam')
        self.shape.serialize(writer)
        self.label.serialize(writer)
    
    def deserialize(self, reader):
        """Deserialize object from HDF5 reader"""
        self.annotationparam = AnnotationParam(_("Annotation"),
                                               icon="annotation.png")
        reader.read('annotationparam', instance=self.annotationparam)
        self.annotationparam.update_annotation(self)
        self.shape.deserialize(reader)
        self.label.deserialize(reader)
    
    def set_style(self, section, option):
        self.shape.set_style(section, option)
        
    #----QwtPlotItem API--------------------------------------------------------
    def draw(self, painter, xMap, yMap, canvasRect):
        self.shape.draw(painter, xMap, yMap, canvasRect)
        if self.label.isVisible():
            self.label.draw(painter, xMap, yMap, canvasRect)
        
    #----Public API-------------------------------------------------------------
    def create_shape(self):
        """Return the shape object associated to this annotated shape object"""
        shape = self.SHAPE_CLASS(0, 0, 1, 1)
        return shape
        
    def create_label(self):
        """Return the label object associated to this annotated shape object"""
        label_param = LabelParam(_("Label"), icon='label.png')
        label_param.read_config(CONF, "plot", "shape/label")
        label_param.anchor = self.LABEL_ANCHOR
        return DataInfoLabel(label_param, [self])
        
    def is_label_visible(self):
        """Return True if associated label is visible"""
        return self.label.isVisible()
        
    def set_label_visible(self, state):
        """Set the annotated shape's label visibility"""
        self.label.setVisible(state)
        
    def update_label(self):
        """Update the annotated shape's label contents"""
        self.label.update_text()

    def get_text(self):
        """
        Return text associated to current shape
        (see :py:class:`guiqwt.label.ObjectInfo`)
        """
        text = ""
        title = self.title().text()
        if title:
            text += "<b>%s</b>" % title
        subtitle = self.annotationparam.subtitle
        if subtitle:
            if text:
                text += "<br>"
            text += "<i>%s</i>" % subtitle
        if self.area_computations_visible:
            infos = self.get_infos()
            if infos:
                if text:
                    text += "<br>"
                text += infos
        return text

    def x_to_str(self, x):
        """Convert x (float) to a string
        (with associated unit and uncertainty)"""
        param = self.annotationparam
        if self.plot() is None:
            return ''
        else:
            xunit = self.plot().get_axis_unit(self.xAxis())
            fmt = param.format
            if param.uncertainty:
                fmt += " ± " + (fmt % (x*param.uncertainty))
            if xunit is not None:
                return (fmt+" "+xunit) % x
            else:
                return (fmt) % x    

    def y_to_str(self, y):
        """Convert y (float) to a string
        (with associated unit and uncertainty)"""
        param = self.annotationparam
        if self.plot() is None:
            return ''
        else:
            yunit = self.plot().get_axis_unit(self.yAxis())
            fmt = param.format
            if param.uncertainty:
                fmt += " ± " + (fmt % (y*param.uncertainty))
            if yunit is not None:
                return (fmt+" "+yunit) % y
            else:
                return (fmt) % y
                
    def get_center(self):
        """Return shape center coordinates: (xc, yc)"""
        return self.shape.get_center()
        
    def get_tr_center(self):
        """Return shape center coordinates after applying transform matrix"""
        raise NotImplementedError
        
    def get_tr_center_str(self):
        """Return center coordinates as a string (with units)"""
        xc, yc = self.get_tr_center()
        return "( %s ; %s )" % (self.x_to_str(xc), self.y_to_str(yc))
        
    def get_tr_size(self):
        """Return shape size after applying transform matrix"""
        raise NotImplementedError
        
    def get_tr_size_str(self):
        """Return size as a string (with units)"""
        xs, ys = self.get_tr_size()
        return "%s x %s" % (self.x_to_str(xs), self.y_to_str(ys))
        
    def get_infos(self):
        """Return formatted string with informations on current shape"""
        pass
        
    def set_label_position(self):
        """Set label position, for instance based on shape position"""
        raise NotImplementedError
    
    def apply_transform_matrix(self, x, y):
        V = np.array([x, y, 1.])
        W = np.dot(V, self.annotationparam.transform_matrix)
        return W[0], W[1]
    
    def get_transformed_coords(self, handle1, handle2):
        x1, y1 = self.apply_transform_matrix(*self.shape.points[handle1])
        x2, y2 = self.apply_transform_matrix(*self.shape.points[handle2])
        return x1, y1, x2, y2

    #----IBasePlotItem API------------------------------------------------------
    def hit_test(self, pos):
        return self.shape.poly_hit_test(self.plot(),
                                        self.xAxis(), self.yAxis(), pos)
            
    def move_point_to(self, handle, pos, ctrl=None):
        self.shape.move_point_to(handle, pos, ctrl)
        self.set_label_position()
        if self.plot():
            self.plot().SIG_ANNOTATION_CHANGED.emit(self)

    def move_shape(self, old_pos, new_pos):
        self.shape.move_shape(old_pos, new_pos)
        self.label.move_local_shape(old_pos, new_pos)
        
    def move_local_shape(self, old_pos, new_pos):
        old_pt = canvas_to_axes(self, old_pos)
        new_pt = canvas_to_axes(self, new_pos)
        self.shape.move_shape(old_pt, new_pt)
        self.set_label_position()
        if self.plot():
            self.plot().SIG_ITEM_MOVED.emit(self, *(old_pt+new_pt))
            self.plot().SIG_ANNOTATION_CHANGED.emit(self)
            
    def move_with_selection(self, delta_x, delta_y):
        """
        Translate the shape together with other selected items
        delta_x, delta_y: translation in plot coordinates
        """
        self.shape.move_with_selection(delta_x, delta_y)
        self.label.move_with_selection(delta_x, delta_y)
        self.plot().SIG_ANNOTATION_CHANGED.emit(self)

    def select(self):
        """Select item"""
        AbstractShape.select(self)
        self.shape.select()
    
    def unselect(self):
        """Unselect item"""
        AbstractShape.unselect(self)
        self.shape.unselect()

    def get_item_parameters(self, itemparams):
        self.shape.get_item_parameters(itemparams)
        self.label.get_item_parameters(itemparams)
        self.annotationparam.update_param(self)
        itemparams.add("AnnotationParam", self, self.annotationparam)
    
    def set_item_parameters(self, itemparams):
        self.shape.set_item_parameters(itemparams)
        self.label.set_item_parameters(itemparams)
        update_dataset(self.annotationparam, itemparams.get("AnnotationParam"),
                       visible_only=True)
        self.annotationparam.update_annotation(self)

assert_interfaces_valid(AnnotatedShape)


class AnnotatedPoint(AnnotatedShape):
    """
    Construct an annotated point at coordinates (x, y) 
    with properties set with *annotationparam* 
    (see :py:class:`guiqwt.styles.AnnotationParam`)
    """
    SHAPE_CLASS = PointShape
    LABEL_ANCHOR = "TL"
    def __init__(self, x=0, y=0, annotationparam=None):
        AnnotatedShape.__init__(self, annotationparam)
        self.set_pos(x, y)
        
    #----Public API-------------------------------------------------------------
    def set_pos(self, x, y):
        """Set the point coordinates to (x, y)"""
        self.shape.set_pos(x, y)
        self.set_label_position()

    def get_pos(self):
        """Return the point coordinates"""
        return self.shape.get_pos()
        
    #----AnnotatedShape API-----------------------------------------------------
    def create_shape(self):
        """Return the shape object associated to this annotated shape object"""
        shape = self.SHAPE_CLASS(0, 0)
        return shape

    def set_label_position(self):
        """Set label position, for instance based on shape position"""
        x, y = self.shape.points[0]
        self.label.set_pos(x, y)
        
    #----AnnotatedShape API-----------------------------------------------------
    def get_infos(self):
        """Return formatted string with informations on current shape"""
        xt, yt = self.apply_transform_matrix(*self.shape.points[0])
        return "( %s ; %s )" % (self.x_to_str(xt), self.y_to_str(yt))
        

class AnnotatedSegment(AnnotatedShape):
    """
    Construct an annotated segment between coordinates (x1, y1) and 
    (x2, y2) with properties set with *annotationparam* 
    (see :py:class:`guiqwt.styles.AnnotationParam`)
    """
    SHAPE_CLASS = SegmentShape
    LABEL_ANCHOR = "C"
    def __init__(self, x1=0, y1=0, x2=0, y2=0, annotationparam=None):
        AnnotatedShape.__init__(self, annotationparam)
        self.set_rect(x1, y1, x2, y2)
        
    #----Public API-------------------------------------------------------------
    def set_rect(self, x1, y1, x2, y2):
        """
        Set the coordinates of the shape's top-left corner to (x1, y1), 
        and of its bottom-right corner to (x2, y2).
        """
        self.shape.set_rect(x1, y1, x2, y2)
        self.set_label_position()

    def get_rect(self):
        """
        Return the coordinates of the shape's top-left and bottom-right corners
        """
        return self.shape.get_rect()
        
    def get_tr_length(self):
        """Return segment length after applying transform matrix"""
        return compute_distance(*self.get_transformed_coords(0, 1))
    
    #----AnnotatedShape API-----------------------------------------------------
    def set_label_position(self):
        """Set label position, for instance based on shape position"""
        x1, y1, x2, y2 = self.get_rect()
        self.label.set_pos(*compute_center(x1, y1, x2, y2))
        
    #----AnnotatedShape API-----------------------------------------------------
    def get_infos(self):
        """Return formatted string with informations on current shape"""
        return _("Distance:") + " " + self.x_to_str(self.get_tr_length())


class AnnotatedRectangle(AnnotatedShape):
    """
    Construct an annotated rectangle between coordinates (x1, y1) and 
    (x2, y2) with properties set with *annotationparam* 
    (see :py:class:`guiqwt.styles.AnnotationParam`)
    """
    SHAPE_CLASS = RectangleShape
    LABEL_ANCHOR = "TL"
    def __init__(self, x1=0, y1=0, x2=0, y2=0, annotationparam=None):
        AnnotatedShape.__init__(self, annotationparam)
        self.set_rect(x1, y1, x2, y2)
        
    #----Public API-------------------------------------------------------------
    def set_rect(self, x1, y1, x2, y2):
        """
        Set the coordinates of the shape's top-left corner to (x1, y1), 
        and of its bottom-right corner to (x2, y2).
        """
        self.shape.set_rect(x1, y1, x2, y2)
        self.set_label_position()

    def get_rect(self):
        """
        Return the coordinates of the shape's top-left and bottom-right corners
        """
        return self.shape.get_rect()
        
    #----AnnotatedShape API-----------------------------------------------------
    def set_label_position(self):
        """Set label position, for instance based on shape position"""
        x_label, y_label = self.shape.points.min(axis=0)
        self.label.set_pos(x_label, y_label)
    
    def get_computations_text(self):
        """Return formatted string with informations on current shape"""
        tdict = self.get_string_dict()
        return "%(center_n)s ( %(center)s )<br>%(size_n)s %(size)s" % tdict
        
    def get_tr_center(self):
        """Return shape center coordinates after applying transform matrix"""
        return compute_center(*self.get_transformed_coords(0, 2))
        
    def get_tr_size(self):
        """Return shape size after applying transform matrix"""
        return compute_rect_size(*self.get_transformed_coords(0, 2))
        
    def get_infos(self):
        """Return formatted string with informations on current shape"""
        return "<br>".join([
                            _("Center:") + " " + self.get_tr_center_str(),
                            _("Size:") + " " + self.get_tr_size_str(),
                            ])


class AnnotatedObliqueRectangle(AnnotatedRectangle):
    """
    Construct an annotated oblique rectangle between coordinates (x0, y0),
    (x1, y1), (x2, y2) and (x3, y3) with properties set with *annotationparam* 
    (see :py:class:`guiqwt.styles.AnnotationParam`)
    """
    SHAPE_CLASS = ObliqueRectangleShape
    LABEL_ANCHOR = "C"
    def __init__(self, x0=0, y0=0, x1=0, y1=0, x2=0, y2=0, x3=0, y3=0,
                 annotationparam=None):
        AnnotatedShape.__init__(self, annotationparam)
        self.set_rect(x0, y0, x1, y1, x2, y2, x3, y3)
        
    #----Public API-------------------------------------------------------------
    def get_tr_angle(self):
        """Return X-diameter angle with horizontal direction,
        after applying transform matrix"""
        xcoords = self.get_transformed_coords(0, 1)
        _x, yr1 = self.apply_transform_matrix(1., 1.)
        _x, yr2 = self.apply_transform_matrix(1., 2.)
        return (compute_angle(reverse=yr1 > yr2, *xcoords)+90)%180-90
        
    def get_bounding_rect_coords(self):
        """Return bounding rectangle coordinates (in plot coordinates)"""
        return self.shape.get_bounding_rect_coords()
        
    #----AnnotatedShape API-----------------------------------------------------
    def create_shape(self):
        """Return the shape object associated to this annotated shape object"""
        shape = self.SHAPE_CLASS(0, 0, 0, 0, 0, 0, 0, 0)
        return shape
        
    #----AnnotatedShape API-----------------------------------------------------
    def set_label_position(self):
        """Set label position, for instance based on shape position"""
        self.label.set_pos(*self.get_center())
        
    #----RectangleShape API-----------------------------------------------------
    def set_rect(self, x0, y0, x1, y1, x2, y2, x3, y3):
        """
        Set the rectangle corners coordinates:
            
            (x0, y0): top-left corner
            (x1, y1): top-right corner
            (x2, y2): bottom-right corner
            (x3, y3): bottom-left corner

        ::
            
            x: additionnal points
            
            (x0, y0)------>(x1, y1)
                ↑             |
                |             |
                x             x
                |             |
                |             ↓
            (x3, y3)<------(x2, y2)
        """
        self.shape.set_rect(x0, y0, x1, y1, x2, y2, x3, y3)
        self.set_label_position()
        
    def get_tr_size(self):
        """Return shape size after applying transform matrix"""
        dx = compute_distance(*self.get_transformed_coords(0, 1))
        dy = compute_distance(*self.get_transformed_coords(0, 3))
        return dx, dy
        
    #----AnnotatedShape API-----------------------------------------------------
    def get_infos(self):
        """Return formatted string with informations on current shape"""
        return "<br>".join([
                            _("Center:") + " " + self.get_tr_center_str(),
                            _("Size:") + " " + self.get_tr_size_str(),
                            _("Angle:") + " %.1f°" % self.get_tr_angle(),
                            ])
    

class AnnotatedEllipse(AnnotatedShape):
    """
    Construct an annotated ellipse with X-axis diameter between 
    coordinates (x1, y1) and (x2, y2) 
    and properties set with *annotationparam* 
    (see :py:class:`guiqwt.styles.AnnotationParam`)
    """
    SHAPE_CLASS = EllipseShape
    LABEL_ANCHOR = "C"
    def __init__(self, x1=0, y1=0, x2=0, y2=0, annotationparam=None):
        AnnotatedShape.__init__(self, annotationparam)
        self.set_xdiameter(x1, y1, x2, y2)

    #----Public API-------------------------------------------------------------
    def set_xdiameter(self, x0, y0, x1, y1):
        """Set the coordinates of the ellipse's X-axis diameter
        Warning: transform matrix is not applied here"""
        self.shape.set_xdiameter(x0, y0, x1, y1)
        self.set_label_position()
                         
    def get_xdiameter(self):
        """Return the coordinates of the ellipse's X-axis diameter
        Warning: transform matrix is not applied here"""
        return self.shape.get_xdiameter()

    def set_ydiameter(self, x2, y2, x3, y3):
        """Set the coordinates of the ellipse's Y-axis diameter
        Warning: transform matrix is not applied here"""
        self.shape.set_ydiameter(x2, y2, x3, y3)
        self.set_label_position()
                         
    def get_ydiameter(self):
        """Return the coordinates of the ellipse's Y-axis diameter
        Warning: transform matrix is not applied here"""
        return self.shape.get_ydiameter()

    def get_rect(self):
        return self.shape.get_rect()
    
    def set_rect(self, x0, y0, x1, y1):
        raise NotImplementedError
        
    def get_tr_angle(self):
        """Return X-diameter angle with horizontal direction,
        after applying transform matrix"""
        xcoords = self.get_transformed_coords(0, 1)
        _x, yr1 = self.apply_transform_matrix(1., 1.)
        _x, yr2 = self.apply_transform_matrix(1., 2.)
        return (compute_angle(reverse=yr1 > yr2, *xcoords)+90)%180-90
        
    #----AnnotatedShape API-----------------------------------------------------
    def set_label_position(self):
        """Set label position, for instance based on shape position"""
        x_label, y_label = self.shape.points.mean(axis=0)
        self.label.set_pos(x_label, y_label)
        
    def get_tr_center(self):
        """Return center coordinates: (xc, yc)"""
        return compute_center(*self.get_transformed_coords(0, 1))
        
    def get_tr_size(self):
        """Return shape size after applying transform matrix"""
        xcoords = self.get_transformed_coords(0, 1)
        ycoords = self.get_transformed_coords(2, 3)
        dx = compute_distance(*xcoords)
        dy = compute_distance(*ycoords)
        if np.fabs(self.get_tr_angle()) > 45:
            dx, dy = dy, dx
        return dx, dy
        
    def get_infos(self):
        """Return formatted string with informations on current shape"""
        return "<br>".join([
                            _("Center:") + " " + self.get_tr_center_str(),
                            _("Size:") + " " + self.get_tr_size_str(),
                            _("Angle:") + " %.1f°" % self.get_tr_angle(),
                            ])
        

class AnnotatedCircle(AnnotatedEllipse):
    """
    Construct an annotated circle with diameter between coordinates 
    (x1, y1) and (x2, y2) and properties set with *annotationparam* 
    (see :py:class:`guiqwt.styles.AnnotationParam`)
    """
    def __init__(self, x1=0, y1=0, x2=0, y2=0, annotationparam=None):
        AnnotatedEllipse.__init__(self, x1, y1, x2, y2, annotationparam)
        
    def get_tr_diameter(self):
        """Return circle diameter after applying transform matrix"""
        return compute_distance(*self.get_transformed_coords(0, 1))
        
    #----AnnotatedShape API-------------------------------------------------
    def get_infos(self):
        """Return formatted string with informations on current shape"""
        return "<br>".join([
                    _("Center:")+" "+self.get_tr_center_str(),
                    _("Diameter:")+" "+self.x_to_str(self.get_tr_diameter()),
                            ])

    #----AnnotatedEllipse API---------------------------------------------------
    def set_rect(self, x0, y0, x1, y1):
        self.shape.set_rect(x0, y0, x1, y1)



# -*- coding: utf-8 -*-
#
# Copyright © 2009-2010 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""Curve fitting tools"""

from __future__ import print_function

SHOW = True # Show test in GUI-based test launcher

import numpy as np

from guiqwt.widgets.fit import FitParam, guifit

def test():
    x = np.linspace(-10, 10, 1000)
    y = np.cos(1.5*x)+np.random.rand(x.shape[0])*.2
    
    def fit(x, params):
        a, b = params
        return np.cos(b*x)+a
    
    a = FitParam("Offset", 1., 0., 2.)
    b = FitParam("Frequency", 2., 1., 10., logscale=True)
    params = [a, b]
    values = guifit(x, y, fit, params, xlabel="Time (s)", ylabel="Power (a.u.)")
    
    print(values)
    print([param.value for param in params])

if __name__ == "__main__":
    test()



# -*- coding: utf-8 -*-
#
# Copyright © 2011-2012 CEA
# Ludovic Aubry
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""Plot computations test"""

SHOW = False # Show test in GUI-based test launcher

def xyimagebug(offset):
    from guiqwt.plot import ImageDialog
    from guiqwt.builder import make
    import numpy
    import guidata
    app = guidata.qapplication()
    data = numpy.random.rand(100, 100)
    x = numpy.arange(100)+offset 
    y = numpy.arange(100)
    image = make.xyimage(x, y, data=data)
    win = ImageDialog()
    plot = win.get_plot()
    plot.add_item(image)
    plot.select_item(image) #this helps in seeing where the image should be
    win.exec_()

if __name__ == '__main__':
    xyimagebug(1e9) #offset=1e9 makes it fail, but offset=1e4 is ok



# -*- coding: utf-8 -*-
#
# Copyright © 2012 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""
fliprotate
----------

The `FlipRotate` module provides a dialog box providing essential GUI elements 
for rotating (arbitrary angle) and cropping an image:
    
    * :py:class:`guiqwt.widgets.fliprotate.FlipRotateDialog`: dialog box
    * :py:class:`guiqwt.widgets.fliprotate.FlipRotateWidget`: equivalent widget

Reference
~~~~~~~~~

.. autoclass:: FlipRotateDialog
   :members:
   :inherited-members:
.. autoclass:: FlipRotateWidget
   :members:
   :inherited-members:
"""

from qtpy.QtWidgets import QLabel, QComboBox

import numpy as np

from guidata.qthelpers import create_toolbutton
from guidata.configtools import get_icon

# Local imports
from guiqwt.config import _
from guiqwt.widgets import base


class FlipRotateMixin(base.BaseTransformMixin):
    """Rotate & Crop mixin class, to be mixed with a class providing the 
    get_plot method, like ImageDialog or FlipRotateWidget (see below)"""

    ROTATION_ANGLES = [str((i - 1) * 90) for i in range(4)]

    # ------BaseTransformMixin API----------------------------------------------
    def add_buttons_to_layout(self, layout):
        """Add tool buttons to layout"""
        # Image orientation
        angle_label = QLabel(_("Angle (°):"))
        layout.addWidget(angle_label)
        self.angle_combo = QComboBox(self)
        self.angle_combo.addItems(self.ROTATION_ANGLES)
        self.angle_combo.setCurrentIndex(1)
        self.angle_combo.currentIndexChanged.connect(
            lambda index: self.apply_transformation()
        )
        layout.addWidget(self.angle_combo)
        layout.addSpacing(10)

        # Image flipping
        flip_label = QLabel(_("Flip:"))
        layout.addWidget(flip_label)
        hflip = create_toolbutton(
            self,
            text="",
            icon=get_icon("hflip.png"),
            toggled=lambda state: self.apply_transformation(),
            autoraise=False,
        )
        self.hflip_btn = hflip
        layout.addWidget(hflip)
        vflip = create_toolbutton(
            self,
            text="",
            icon=get_icon("vflip.png"),
            toggled=lambda state: self.apply_transformation(),
            autoraise=False,
        )
        self.vflip_btn = vflip
        layout.addWidget(vflip)
        layout.addSpacing(15)

        self.add_reset_button(layout)

    def reset_transformation(self):
        """Reset transformation"""
        self.angle_combo.setCurrentIndex(1)
        self.hflip_btn.setChecked(False)
        self.vflip_btn.setChecked(False)

    def apply_transformation(self):
        """Apply transformation, e.g. crop or rotate"""
        angle, hflip, vflip = self.get_parameters()
        x, y, _a, px, py, _hf, _vf = self.item.get_transform()
        self.item.set_transform(x, y, angle * np.pi / 180, px, py, hflip, vflip)
        self.get_plot().replot()

    def compute_transformation(self):
        """Compute transformation, return compute output array"""
        angle, hflip, vflip = self.get_parameters()
        data = self.item.data.copy()
        if hflip:
            data = np.fliplr(data)
        if vflip:
            data = np.flipud(data)
        if angle:
            k = int((-angle % 360) / 90)
            data = np.rot90(data, k)
        return data

    # ------Public API----------------------------------------------------------
    def get_parameters(self):
        """Return transform parameters"""
        angle = int(str(self.angle_combo.currentText()))
        hflip = self.hflip_btn.isChecked()
        vflip = self.vflip_btn.isChecked()
        return angle, hflip, vflip

    def set_parameters(self, angle, hflip, vflip):
        """Set transform parameters"""
        angle_index = self.ROTATION_ANGLES.index(str(angle))
        self.angle_combo.setCurrentIndex(angle_index)
        self.hflip_btn.setChecked(hflip)
        self.vflip_btn.setChecked(vflip)


class FlipRotateDialog(base.BaseTransformDialog, FlipRotateMixin):
    """Flip & Rotate Dialog
    
    Flip and rotate a :py:class:`guiqwt.image.TrImageItem` plot item"""

    def __init__(self, parent, wintitle=None, options=None, resize_to=None):
        FlipRotateMixin.__init__(self)
        base.BaseTransformDialog.__init__(
            self, parent, wintitle=wintitle, options=options, resize_to=resize_to
        )


class FlipRotateWidget(base.BaseTransformWidget, FlipRotateMixin):
    """Flip & Rotate Widget
    
    Flip and rotate a :py:class:`guiqwt.image.TrImageItem` plot item"""

    def __init__(self, parent, options=None):
        base.BaseTransformWidget.__init__(self, parent, options=options)
        FlipRotateMixin.__init__(self)


class MultipleFlipRotateWidget(base.BaseMultipleTransformWidget):
    """Multiple Flip & Rotate Widget
    
    Flip and rotate several :py:class:`guiqwt.image.TrImageItem` plot items"""

    TRANSFORM_WIDGET_CLASS = FlipRotateWidget



# -*- coding: utf-8 -*-
#
# Copyright © 2012 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""Flip/rotate test"""

SHOW = True # Show test in GUI-based test launcher

from guiqwt.widgets.fliprotate import FlipRotateDialog, FlipRotateWidget
from guiqwt.tests.rotatecrop import imshow, create_test_data

def widget_test(fname, qapp):
    """Test the rotate/crop widget"""
    array0, item = create_test_data(fname)
    widget = FlipRotateWidget(None)
    widget.set_item(item)
    widget.set_parameters(-90, True, False)
    widget.show()
    qapp.exec_()

def dialog_test(fname, interactive=True):
    """Test the rotate/crop dialog"""
    array0, item = create_test_data(fname)
    dlg = FlipRotateDialog(None)
    dlg.set_item(item)
    if dlg.exec_():
        array1 = dlg.output_array
        imshow(array0, title="array0", hold=True)
        imshow(array1, title="array1")


if __name__ == '__main__':
    from guidata import qapplication
    qapp = qapplication()  # analysis:ignore
    
    widget_test("brain.png", qapp)
    dialog_test(fname="brain.png", interactive=True)
    


# -*- coding: utf-8 -*-
#
# Copyright © 2009-2011 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""
guiqwt.transitional
-------------------

The purpose of this transitional package is to regroup all the references to 
the ``PythonQwt`` library (`qwt` package).

No other ``guiqwt`` module should import ``qwt`` or use any of its 
interfaces directly.
"""

from qwt import (QwtPlot, QwtSymbol, QwtLinearScaleEngine, QwtLogScaleEngine,
                 QwtText, QwtPlotCanvas, QwtLinearColorMap, QwtInterval,
                 toQImage, QwtPlotGrid, QwtPlotItem, QwtScaleMap, QwtPlotCurve,
                 QwtPlotMarker, QwtPlotRenderer)



# -*- coding: utf-8 -*-
#
# Copyright © 2011 CEA
# Ludovic Aubry
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""PolygonMapItem test

PolygonMapItem is intended to display maps ie items containing
several hundreds of independent polygons.
"""

from __future__ import print_function

SHOW = True # Show test in GUI-based test launcher

from guiqwt.plot import ImageDialog
from guiqwt.curve import PolygonMapItem

from numpy.random import rand, randint
from numpy import (concatenate, linspace, int32, uint32, zeros, empty,
                   pi, cos, sin)


# Create a sample dataset consisting of tesselated circles randomly placed
# in a box
RMAX=.5
XMAX=YMAX=10.
NSEGMIN=4
NSEGMAX=300

def create_circle():
    x, y, rmax = rand(3)
    rmax*=RMAX
    x*=XMAX
    y*=YMAX
    nseg = randint(NSEGMIN, NSEGMAX)
    th = linspace(0, 2*pi, nseg)
    PTS = empty( (nseg, 2), float)
    PTS[:, 0] = x+rmax*cos(th)
    PTS[:, 1] = y+rmax*sin(th)
    return PTS

NCIRC=1000
COLORS=[
    (0xff000000, 0x8000ff00),
    (0xff0000ff, 0x800000ff),
    (0xff000000, 0x80ff0000),
    (0xff00ff00, 0x80000000),
]


def test():
    win = ImageDialog(edit=True, toolbar=True,
                      wintitle="Sample multi-polygon item")
    plot = win.get_plot()
    plot.set_aspect_ratio(lock=True)
    plot.set_antialiasing(False)
    plot.set_axis_direction('left', False)
    plot.set_axis_title("bottom", "Lon")
    plot.set_axis_title("left", "Lat")
    
    points = []
    offsets = zeros( (NCIRC, 2), int32)
    colors = zeros( (NCIRC, 2), uint32)
    npts = 0
    for k in range(NCIRC):
        pts = create_circle()
        offsets[k, 0] = k
        offsets[k, 1] = npts
        npts += pts.shape[0]
        points.append(pts)
        colors[k, 0] = COLORS[k%len(COLORS)][0]
        colors[k, 1] = COLORS[(3*k)%len(COLORS)][1]
    points = concatenate(points)
    
    print(NCIRC, "Polygons")
    print(points.shape[0], "Points")
    
    crv = PolygonMapItem()
    crv.set_data(points, offsets, colors)
    plot.add_item(crv, z=0)
    win.show()
    win.exec_()

if __name__ == '__main__':
    import guidata
    app = guidata.qapplication()
    test()



# -*- coding: utf-8 -*-
#
# Copyright © 2012 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""Shows how to customize a shape created with a tool like RectangleTool"""

SHOW = True # Show test in GUI-based test launcher

import os.path as osp

from guiqwt.plot import ImageDialog
from guiqwt.tools import (RectangleTool, EllipseTool, SegmentTool,
                          MultiLineTool, FreeFormTool)
from guiqwt.builder import make
from guiqwt.styles import style_generator, update_style_attr

STYLE = style_generator()

def customize_shape(shape):
    global STYLE
    param = shape.shapeparam
    style = next(STYLE)
    update_style_attr(style, param)
    param.update_shape(shape)
    shape.plot().replot()

def create_window():
    gridparam = make.gridparam(background="black",
                               minor_enabled=(False, False),
                               major_style=(".", "gray", 1))
    win = ImageDialog(edit=False, toolbar=True,
                      wintitle="All image and plot tools test",
                      options=dict(gridparam=gridparam))
    for toolklass in (RectangleTool, EllipseTool, SegmentTool,
                      MultiLineTool, FreeFormTool):
        win.add_tool(toolklass, handle_final_shape_cb=customize_shape)
    return win

def test():
    """Test"""
    # -- Create QApplication
    import guidata
    _app = guidata.qapplication()
    # --
    filename = osp.join(osp.dirname(__file__), "brain.png")
    win = create_window()
    image = make.image(filename=filename, colormap="bone", alpha_mask=True)
    plot = win.get_plot()
    plot.add_item(image)
    win.exec_()

if __name__ == "__main__":
    test()



# -*- coding: utf-8 -*-
#
# Copyright © 2009-2010 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""
SelectPointTool test

This guiqwt tool provide a MATLAB-like "ginput" feature.
"""

from __future__ import print_function

SHOW = True # Show test in GUI-based test launcher

from guiqwt.plot import CurveDialog
from guiqwt.tools import SelectPointTool
from guiqwt.builder import make
from guiqwt.config import _


def test_function(tool):
    print("Current coordinates:", tool.get_coordinates())

def get_point( *args ):
    """
    Plot curves and return selected point(s) coordinates
    """
    win = CurveDialog(_("Select one point then press OK to accept"), edit=True)
    default = win.add_tool(SelectPointTool, title="Test", on_active_item=True,
                           mode="create", end_callback=test_function)
    default.activate()
    plot = win.get_plot()
    for cx, cy in args:
        item = make.mcurve(cx, cy)
        plot.add_item(item)
    plot.set_active_item(item)
    win.show()
    if win.exec_():
        return default.get_coordinates()

def test():
    """Test"""
    # -- Create QApplication
    import guidata
    _app = guidata.qapplication()
    # --
    from numpy import linspace, sin
    x = linspace(-10, 10, 1000)
    y = sin(sin(sin(x)))    
    x2 = linspace(-10, 10, 20)
    y2 = sin(sin(sin(x2)))
    print(get_point( (x, y), (x2, y2), (x, sin(2*y)) ))

if __name__ == "__main__":
    test()



# -*- coding: utf-8 -*-
#
# Copyright © 2009-2015 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""
guiqwt
======

Copyright © 2009-2015 CEA
Pierre Raybaut
Licensed under the terms of the CECILL License
(see guiqwt/__init__.py for details)
"""

# Building extensions:
# python setup.py build_ext -c mingw32 --inplace

from __future__ import print_function

import setuptools  # analysis:ignore
import numpy
import sys
import os
import os.path as osp
import subprocess
from numpy.distutils.core import setup, Extension
from guidata.utils import get_subpackages, get_package_data, cythonize_all

# TODO: copy qtdesigner plugins in Lib\site-packages\PyQt4\plugins\designer\python
#      note: this directory doesn't exist for a default PyQt4 install


LIBNAME = "guiqwt"
from guiqwt import __version__
from guiqwt import __description__

# Remove module from list to allow building doc from build dir
del sys.modules["guiqwt"]

LONG_DESCRIPTION = """\
guiqwt: Python tools for curve and image plotting
=================================================

.. image:: https://raw.githubusercontent.com/PierreRaybaut/guiqwt/master/doc/images/panorama.png

See `documentation`_ for more details on the library and `changelog`_ for 
recent history of changes.

Copyright © 2009-2020 CEA, Pierre Raybaut, licensed under the terms of the 
`CECILL License`_.

.. _documentation: https://guiqwt.readthedocs.io/en/latest/
.. _changelog: https://github.com/PierreRaybaut/guiqwt/blob/master/CHANGELOG.md
.. _CECILL License: https://github.com/PierreRaybaut/guiqwt/blob/master/Licence_CeCILL_V2-en.txt


Overview
--------

Based on `PythonQwt`_ (a pure Python/PyQt reimplementation of the curve 
plotting Qwt C++ library, included in guiqwt base source code) and on the 
scientific modules NumPy and SciPy, ``guiqwt`` is a Python library providing 
efficient 2D data-plotting features (curve/image visualization and related 
tools) for interactive computing and signal/image processing application 
development. It is based on Qt graphical user interfaces library, and 
currently supports both ``PyQt4`` and ``PyQt5``.

Extension to `PythonQwt`_:

* set of tools for curve and image plotting
* GUI-based application development helpers

.. _PythonQwt: https://pypi.python.org/pypi/PythonQwt


Building, installation, ...
---------------------------

The following packages are **required**: `PyQt4`_ (or `PyQt5`_), 
`PythonQwt`_, `guidata`_, `NumPy`_, `SciPy`_ and `Pillow`_.

.. _PyQt4: https://pypi.python.org/pypi/PyQt4
.. _PyQt5: https://pypi.python.org/pypi/PyQt5
.. _PythonQwt: https://pypi.python.org/pypi/PythonQwt
.. _guidata: https://pypi.python.org/pypi/guidata
.. _NumPy: https://pypi.python.org/pypi/NumPy
.. _SciPy: https://pypi.python.org/pypi/SciPy
.. _Pillow: https://pypi.python.org/pypi/Pillow

See the `README`_ and `documentation`_ for more details.

.. _README: https://github.com/PierreRaybaut/guiqwt/blob/master/README.md
"""

KEYWORDS = ""
CLASSIFIERS = ["Topic :: Scientific/Engineering"]
if "beta" in __version__ or "b" in __version__:
    CLASSIFIERS += ["Development Status :: 4 - Beta"]
elif "alpha" in __version__ or "a" in __version__:
    CLASSIFIERS += ["Development Status :: 3 - Alpha"]
else:
    CLASSIFIERS += ["Development Status :: 5 - Production/Stable"]


def build_chm_doc(libname):
    """Return CHM documentation file (on Windows only), which is copied under 
    {PythonInstallDir}\Doc, hence allowing Spyder to add an entry for opening 
    package documentation in "Help" menu. This has no effect on a source 
    distribution."""
    args = "".join(sys.argv)
    if "--no-doc" in sys.argv:
        sys.argv.remove("--no-doc")
        return
    if (
        os.name == "nt"
        and ("bdist" in args or "build" in args)
        and "--inplace" not in args
    ):
        try:
            import sphinx  # analysis:ignore
        except ImportError:
            print(
                "Warning: `sphinx` is required to build documentation", file=sys.stderr
            )
            return
        hhc_base = r"C:\Program Files%s\HTML Help Workshop\hhc.exe"
        for hhc_exe in (hhc_base % "", hhc_base % " (x86)"):
            if osp.isfile(hhc_exe):
                break
        else:
            print(
                "Warning: `HTML Help Workshop` is required to build CHM "
                "documentation file",
                file=sys.stderr,
            )
            return
        doctmp_dir = osp.join("build", "doctmp")
        if not osp.isdir(doctmp_dir):
            os.makedirs(doctmp_dir)
        fname = osp.abspath(osp.join(doctmp_dir, "%s.chm" % libname))
        if osp.isfile(fname):
            # doc has already been built
            return fname
        subprocess.call("sphinx-build -b htmlhelp doc %s" % doctmp_dir, shell=True)
        subprocess.call('"%s" %s' % (hhc_exe, fname), shell=True)
        if osp.isfile(fname):
            return fname
        else:
            print("Warning: CHM building process failed", file=sys.stderr)


CHM_DOC = build_chm_doc(LIBNAME)


def _create_script_list(basename):
    scripts = ["%s-py%d" % (basename, sys.version_info.major)]
    if os.name == "nt":
        scripts.append("%s.bat" % scripts[0])
    return [osp.join("scripts", name) for name in scripts]


SCRIPTS = _create_script_list("guiqwt-tests") + _create_script_list("sift")


try:
    import sphinx
except ImportError:
    sphinx = None  # analysis:ignore


def is_msvc():
    """Detect if Microsoft Visual C++ compiler was chosen to build package"""
    # checking if mingw is the compiler
    # mingw32 compiler configured in %USERPROFILE%\pydistutils.cfg
    # or distutils\distutils.cfg
    from distutils.dist import Distribution

    dist = Distribution()
    dist.parse_config_files()
    bld = dist.get_option_dict("build")
    if bld:
        comp = bld.get("compiler")
        if comp is not None and "mingw32" in comp:
            return False  # mingw is the compiler
    return os.name == "nt" and "mingw" not in "".join(sys.argv)


CFLAGS = ["-Wall"]
if is_msvc():
    CFLAGS.insert(0, "/EHsc")
for arg, compile_arg in (
    ("--sse2", "-msse2"),
    ("--sse3", "-msse3"),
):
    if arg in sys.argv:
        sys.argv.pop(sys.argv.index(arg))
        CFLAGS.insert(0, compile_arg)

# Compiling Cython modules to C source code: this is the only way I found to
# be able to build both Fortran and Cython extensions together
# (this could be changed now as there is no longer Fortran extensions here...)
cythonize_all("src")

setup(
    name=LIBNAME,
    version=__version__,
    description=__description__,
    long_description=LONG_DESCRIPTION,
    packages=get_subpackages(LIBNAME),
    package_data={
        LIBNAME: get_package_data(LIBNAME, (".png", ".svg", ".mo", ".dcm", ".ui"))
    },
    data_files=[(r"Doc", [CHM_DOC])] if CHM_DOC else [],
    install_requires=[
        "NumPy>=1.3",
        "SciPy>=0.7",
        "guidata>=1.7.0",
        "PythonQwt>=0.5.0",
        "Pillow",
        "QtPy>=1.3",
    ],
    extras_require={"Doc": ["Sphinx>=1.1"], "DICOM": ["pydicom>=0.9.3"],},
    entry_points={
        "gui_scripts": [
            "guiqwt-tests-py%d = guiqwt.tests:run" % sys.version_info.major,
            "sift-py%d = guiqwt.tests.sift:run" % sys.version_info.major,
        ]
    },
    ext_modules=[
        Extension(
            LIBNAME + ".histogram2d",
            [osp.join("src", "histogram2d.c")],
            include_dirs=[numpy.get_include()],
        ),
        Extension(
            LIBNAME + ".mandelbrot",
            [osp.join("src", "mandelbrot.c")],
            include_dirs=[numpy.get_include()],
        ),
        Extension(
            LIBNAME + "._scaler",
            [osp.join("src", "scaler.cpp"), osp.join("src", "pcolor.cpp")],
            extra_compile_args=CFLAGS,
            depends=[
                osp.join("src", "traits.hpp"),
                osp.join("src", "points.hpp"),
                osp.join("src", "arrays.hpp"),
                osp.join("src", "scaler.hpp"),
                osp.join("src", "debug.hpp"),
            ],
        ),
    ],
    author="Pierre Raybaut",
    author_email="pierre.raybaut@gmail.com",
    url="https://github.com/PierreRaybaut/%s" % LIBNAME,
    license="CeCILL V2",
    classifiers=CLASSIFIERS
    + [
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: OS Independent",
        "Operating System :: POSIX",
        "Operating System :: Unix",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
    ],
)




# -*- coding: utf-8 -*-
#
# Copyright © 2009-2010 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""
AnnotatedSegmentTool test

This guiqwt tool provide a MATLAB-like "ginput" feature.
"""

from __future__ import print_function

SHOW = True # Show test in GUI-based test launcher

import os.path as osp
import numpy as np

from guiqwt.plot import ImageDialog
from guiqwt.tools import SelectTool, AnnotatedSegmentTool
from guiqwt.builder import make
from guiqwt.config import _


def get_segment(item):
    """Show image and return selected segment coordinates"""
    win = ImageDialog(_("Select a segment then press OK to accept"), edit=True)
    default = win.add_tool(SelectTool)
    win.set_default_tool(default)
    segtool = win.add_tool(AnnotatedSegmentTool, title="Test",
                           switch_to_default_tool=True)
    segtool.activate()
    plot = win.get_plot()
    plot.add_item(item)
    plot.set_active_item(item)
    win.show()
    if win.exec_():
        shape = segtool.get_last_final_shape()
        return shape.get_rect()

def test():
    """Test"""
    # -- Create QApplication
    import guidata
    _app = guidata.qapplication()
    # --
    filename = osp.join(osp.dirname(__file__), "brain.png")
    image = make.image(filename=filename, colormap="bone")
    rect = get_segment(image)
    print("Coordinates:", rect)
    print("Distance:", np.sqrt((rect[2]-rect[0])**2 + (rect[3]-rect[1])**2))

if __name__ == "__main__":
    test()



# -*- coding: utf-8 -*-
#
# Copyright © 2009-2010 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""
guiqwt.debug
------------

The `debug` module contains some debugging functions (mostly dumping attributes
of Qt Objects).
"""

from __future__ import print_function

from qtpy.QtGui import QImage, QInputEvent
from qtpy.QtCore import Qt, QEvent

from qtpy.py3compat import io


def buttons_to_str(buttons):
    """Conversion des flags Qt en chaine"""
    string = ""
    if buttons & Qt.LeftButton:
        string += "L"
    if buttons & Qt.MidButton:
        string += "M"
    if buttons & Qt.RightButton:
        string += "R"
    return string


def evt_type_to_str(type):
    """Représentation textuelle d'un type d'événement (debug)"""
    if type == QEvent.MouseButtonPress:
        return "Mpress"
    elif type == QEvent.MouseButtonRelease:
        return "Mrelease"
    elif type == QEvent.MouseMove:
        return "Mmove"
    elif type == QEvent.ContextMenu:
        return "Context"
    else:
        return "%d" % type


def print_event(evt):
    """Représentation textuelle d'un événement (debug)"""
    string = ""
    if isinstance(evt, QInputEvent):
        string += evt_type_to_str(evt.type())
        string += "%08x:" % evt.modifiers()
        if hasattr(evt, "buttons"):
            buttons = evt.buttons()
        elif hasattr(evt, "buttons"):
            buttons = evt.button()
        else:
            buttons = 0
        string += buttons_to_str(buttons)
    if string:
        print(string)
    else:
        print(evt)


def qimage_format(fmt):
    for attr in dir(QImage):
        if attr.startswith("Format"):
            val = getattr(QImage, attr)
            if val == fmt:
                return attr[len("Format_") :]
    return str(fmt)


def qimage_to_str(img, indent=""):
    fd = io.StringIO()
    print(indent, img, file=fd)
    indent += "  "
    print(indent, "Size:", img.width(), "x", img.height(), file=fd)
    print(indent, "Depth:", img.depth(), file=fd)
    print(indent, "Format", qimage_format(img.format()), file=fd)
    return fd.getvalue()



# -*- coding: utf-8 -*-
#
# Copyright © 2009-2010 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""
guiqwt.config
-------------

The `config` module handles `guiqwt` configuration (options, images and icons).
"""

import os.path as osp

from guidata.configtools import add_image_module_path, get_translation


_ = get_translation( "guiqwt" )


def make_title(basename, count):
    """Make item title with *basename* and *count* number"""
    return "%s %s%d" % (basename, _("#"), count)


APP_PATH = osp.dirname(__file__)
add_image_module_path( "guiqwt", "images" )

DEFAULTS = {
            'plot':
             {
              "selection/distance": 6,
              "antialiasing": False,
              
              "title/font/size": 12,
              "title/font/bold": False,

              "selected_curve_symbol/marker": 'Rect',
              "selected_curve_symbol/edgecolor": "gray",
              "selected_curve_symbol/facecolor": "black",
              "selected_curve_symbol/alpha": .3,
              "selected_curve_symbol/size": 7,

              # Default parameters for plot axes
              "axis/title": "",
              "axis/unit": "",
              "axis/color": "black",
              "axis/title_font/size": 8,
              "axis/title_font/family": "default",
              "axis/title_font/bold": False,
              "axis/title_font/italic": False,
              "axis/ticks_font/size": 8,
              "axis/ticks_font/family": "default",
              "axis/ticks_font/bold": False,
              "axis/ticks_font/italic": False,

              # Default parameters for X/Y image axes
              "image_axis/title": "",
              "image_axis/unit": _("pixels"),
              "image_axis/color": "black",
              "image_axis/title_font/size": 8,
              "image_axis/title_font/family": "default",
              "image_axis/title_font/bold": False,
              "image_axis/title_font/italic": False,
              "image_axis/ticks_font/size": 8,
              "image_axis/ticks_font/family": "default",
              "image_axis/ticks_font/bold": False,
              "image_axis/ticks_font/italic": False,

              # Default parameters for color scale
              "color_axis/title": _("Intensity"),
              "color_axis/unit": _("lsb"),
              "color_axis/color": "black",
              "color_axis/title_font/size": 8,
              "color_axis/title_font/family": "default",
              "color_axis/title_font/bold": False,
              "color_axis/title_font/italic": False,
              "color_axis/ticks_font/size": 8,
              "color_axis/ticks_font/family": "default",
              "color_axis/ticks_font/bold": False,
              "color_axis/ticks_font/italic": False,

              "grid/maj_xenabled": True,
              "grid/maj_yenabled": True,
              "grid/maj_line/color": "darkgray",
              "grid/maj_line/width": 1,
              "grid/maj_line/style": 'DotLine',
              "grid/min_xenabled": True,
              "grid/min_yenabled": True,
              "grid/min_line/color": "#eaeaea",
              "grid/min_line/width": 1,
              "grid/min_line/style": 'DotLine',
              
              "marker/curve/symbol/marker": 'Rect',
              "marker/curve/symbol/edgecolor": "blue",
              "marker/curve/symbol/facecolor": "cyan",
              "marker/curve/symbol/alpha": .8,
              "marker/curve/symbol/size": 7,
              "marker/curve/text/font/size": 8,
              "marker/curve/text/font/family": "default",
              "marker/curve/text/font/bold": False,
              "marker/curve/text/font/italic": False,
              "marker/curve/text/textcolor": "black",
              "marker/curve/text/background_color": "#ffffff",
              "marker/curve/text/background_alpha": 0.8,
              "marker/curve/line/style": "SolidLine",
              "marker/curve/line/color": "black",
              "marker/curve/line/width": 0,
              "marker/curve/sel_symbol/marker": 'Rect',
              "marker/curve/sel_symbol/edgecolor": "blue",
              "marker/curve/sel_symbol/facecolor": "cyan",
              "marker/curve/sel_symbol/alpha": .8,
              "marker/curve/sel_symbol/size": 7,
              "marker/curve/sel_text/font/size": 8,
              "marker/curve/sel_text/font/family": "default",
              "marker/curve/sel_text/font/bold": False,
              "marker/curve/sel_text/font/italic": False,
              "marker/curve/sel_text/textcolor": "black",
              "marker/curve/sel_text/background_color": "#ffffff",
              "marker/curve/sel_text/background_alpha": 0.8,
              "marker/curve/sel_line/style": "SolidLine",
              "marker/curve/sel_line/color": "black",
              "marker/curve/sel_line/width": 0,
              "marker/curve/markerstyle": "NoLine",
              "marker/curve/spacing": 7,

              "marker/cross/symbol/marker": 'Cross',
              "marker/cross/symbol/edgecolor": "black",
              "marker/cross/symbol/facecolor": "red",
              "marker/cross/symbol/alpha": 1.,
              "marker/cross/symbol/size": 8,
              "marker/cross/text/font/family": "default",
              "marker/cross/text/font/size": 8,
              "marker/cross/text/font/bold": False,
              "marker/cross/text/font/italic": False,
              "marker/cross/text/textcolor": "black",
              "marker/cross/text/background_color": "#ffffff",
              "marker/cross/text/background_alpha": 0.8,
              "marker/cross/line/style": "DashLine",
              "marker/cross/line/color": "yellow",
              "marker/cross/line/width": 1,
              "marker/cross/sel_symbol/marker": 'Cross',
              "marker/cross/sel_symbol/edgecolor": "black",
              "marker/cross/sel_symbol/facecolor": "red",
              "marker/cross/sel_symbol/alpha": 1.,
              "marker/cross/sel_symbol/size": 8,
              "marker/cross/sel_text/font/family": "default",
              "marker/cross/sel_text/font/size": 8,
              "marker/cross/sel_text/font/bold": False,
              "marker/cross/sel_text/font/italic": False,
              "marker/cross/sel_text/textcolor": "black",
              "marker/cross/sel_text/background_color": "#ffffff",
              "marker/cross/sel_text/background_alpha": 0.8,
              "marker/cross/sel_line/style": "DashLine",
              "marker/cross/sel_line/color": "yellow",
              "marker/cross/sel_line/width": 1,
              "marker/cross/markerstyle": "Cross",
              "marker/cross/spacing": 7,

              "marker/selectpoint/symbol/marker": 'Rect',
              "marker/selectpoint/symbol/edgecolor": "blue",
              "marker/selectpoint/symbol/facecolor": "cyan",
              "marker/selectpoint/symbol/alpha": .8,
              "marker/selectpoint/symbol/size": 7,
              "marker/selectpoint/text/font/size": 8,
              "marker/selectpoint/text/font/family": "default",
              "marker/selectpoint/text/font/bold": False,
              "marker/selectpoint/text/font/italic": False,
              "marker/selectpoint/text/textcolor": "black",
              "marker/selectpoint/text/background_color": "#ffffff",
              "marker/selectpoint/text/background_alpha": 0.8,
              "marker/selectpoint/line/style": "SolidLine",
              "marker/selectpoint/line/color": "black",
              "marker/selectpoint/line/width": 0,
              "marker/selectpoint/sel_symbol/marker": 'Rect',
              "marker/selectpoint/sel_symbol/edgecolor": "blue",
              "marker/selectpoint/sel_symbol/facecolor": "cyan",
              "marker/selectpoint/sel_symbol/alpha": .8,
              "marker/selectpoint/sel_symbol/size": 7,
              "marker/selectpoint/sel_text/font/size": 8,
              "marker/selectpoint/sel_text/font/family": "default",
              "marker/selectpoint/sel_text/font/bold": False,
              "marker/selectpoint/sel_text/font/italic": False,
              "marker/selectpoint/sel_text/textcolor": "black",
              "marker/selectpoint/sel_text/background_color": "#ffffff",
              "marker/selectpoint/sel_text/background_alpha": 0.8,
              "marker/selectpoint/sel_line/style": "SolidLine",
              "marker/selectpoint/sel_line/color": "black",
              "marker/selectpoint/sel_line/width": 0,
              "marker/selectpoint/markerstyle": "NoLine",
              "marker/selectpoint/spacing": 7,

              "marker/cursor/line/style": "SolidLine",
              "marker/cursor/line/color": "#ff9393",
              "marker/cursor/line/width": 1.,
              "marker/cursor/symbol/marker": 'Ellipse',
              "marker/cursor/symbol/size": 7,
              "marker/cursor/symbol/edgecolor": "white",
              "marker/cursor/symbol/facecolor": "#ff9393",
              "marker/cursor/symbol/alpha": 1.,
              "marker/cursor/text/font/size": 8,
              "marker/cursor/text/font/family": "default",
              "marker/cursor/text/font/bold": False,
              "marker/cursor/text/font/italic": False,
              "marker/cursor/text/textcolor": "#ff9393",
              "marker/cursor/text/background_color": "#ffffff",
              "marker/cursor/text/background_alpha": 0.8,
              "marker/cursor/sel_line/style": "SolidLine",
              "marker/cursor/sel_line/color": "red",
              "marker/cursor/sel_line/width": 2.,
              "marker/cursor/sel_symbol/marker": 'Ellipse',
              "marker/cursor/sel_symbol/size": 9,
              "marker/cursor/sel_symbol/edgecolor": "white",
              "marker/cursor/sel_symbol/facecolor": "red",
              "marker/cursor/sel_symbol/alpha": .9,
              "marker/cursor/sel_text/font/size": 8,
              "marker/cursor/sel_text/font/family": "default",
              "marker/cursor/sel_text/font/bold": True,
              "marker/cursor/sel_text/font/italic": False,
              "marker/cursor/sel_text/textcolor": "red",
              "marker/cursor/sel_text/background_color": "#ffffff",
              "marker/cursor/sel_text/background_alpha": 0.8,
              "marker/cursor/markerstyle": "NoLine",
              "marker/cursor/spacing": 7,

              "shape/drag/line/style": 'SolidLine',
              "shape/drag/line/color": "#ffff00",
              "shape/drag/line/width": 1,
              "shape/drag/fill/style": "SolidPattern",
              "shape/drag/fill/color": "white",
              "shape/drag/fill/alpha": 0.1,              
              "shape/drag/symbol/marker": 'Rect',
              "shape/drag/symbol/size": 3,
              "shape/drag/symbol/edgecolor": "#ffff00",
              "shape/drag/symbol/facecolor": "#ffff00",
              "shape/drag/symbol/alpha": 1.,
              "shape/drag/sel_line/style": 'SolidLine',
              "shape/drag/sel_line/color": "#00ff00",
              "shape/drag/sel_line/width": 1,
              "shape/drag/sel_fill/style": "SolidPattern",
              "shape/drag/sel_fill/color": "white",
              "shape/drag/sel_fill/alpha": 0.1,              
              "shape/drag/sel_symbol/marker": 'Rect',
              "shape/drag/sel_symbol/size": 9,
              "shape/drag/sel_symbol/edgecolor": "#00aa00",
              "shape/drag/sel_symbol/facecolor": "#00ff00",
              "shape/drag/sel_symbol/alpha": .7,

              "shape/imageborder/line/style": 'NoPen',
              "shape/imageborder/line/color": "gray",
              "shape/imageborder/line/width": 0,
              "shape/imageborder/fill/style": "NoBrush",
              "shape/imageborder/fill/color": "black",
              "shape/imageborder/fill/alpha": 0.0,              
              "shape/imageborder/fill/angle": 0.0,
              "shape/imageborder/fill/sx": 1.0,
              "shape/imageborder/fill/sy": 1.0,
              "shape/imageborder/symbol/marker": 'NoSymbol',
              "shape/imageborder/symbol/size": 7,
              "shape/imageborder/symbol/edgecolor": "gray",
              "shape/imageborder/symbol/facecolor": "black",
              "shape/imageborder/symbol/alpha": .3,
              "shape/imageborder/sel_line/style": 'SolidLine',
              "shape/imageborder/sel_line/color": "gray",
              "shape/imageborder/sel_line/width": 3,
              "shape/imageborder/sel_symbol/marker": 'Rect',
              "shape/imageborder/sel_symbol/size": 7,
              "shape/imageborder/sel_symbol/edgecolor": "gray",
              "shape/imageborder/sel_symbol/facecolor": "black",
              "shape/imageborder/sel_symbol/alpha": .8,
              "shape/imageborder/sel_fill/style": "NoBrush",
              "shape/imageborder/sel_fill/color": "gray",
              "shape/imageborder/sel_fill/alpha": 0.5,
              "shape/imageborder/sel_fill/angle": 0.0,
              "shape/imageborder/sel_fill/sx": 1.0,
              "shape/imageborder/sel_fill/sy": 1.0,

              "shape/imagefilter/line/style": 'SolidLine',
              "shape/imagefilter/line/color": "#ffff00",
              "shape/imagefilter/line/width": 1,
              "shape/imagefilter/sel_line/style": 'SolidLine',
              "shape/imagefilter/sel_line/color": "#00ffff",
              "shape/imagefilter/sel_line/width": 2,
              "shape/imagefilter/fill/style": "NoBrush",
              "shape/imagefilter/fill/color": "white",
              "shape/imagefilter/fill/alpha": 0.0,
              "shape/imagefilter/sel_fill/style": "SolidPattern",
              "shape/imagefilter/sel_fill/color": "white",
              "shape/imagefilter/sel_fill/alpha": 0.2,
              "shape/imagefilter/symbol/marker": 'Rect',
              "shape/imagefilter/symbol/size": 3,
              "shape/imagefilter/symbol/edgecolor": "#ffff00",
              "shape/imagefilter/symbol/facecolor": "#ffff00",
              "shape/imagefilter/symbol/alpha": 1.,
              "shape/imagefilter/sel_symbol/marker": 'Ellipse',
              "shape/imagefilter/sel_symbol/size": 7,
              "shape/imagefilter/sel_symbol/edgecolor": "#0000ff",
              "shape/imagefilter/sel_symbol/facecolor": "#00ffff",
              "shape/imagefilter/sel_symbol/alpha": .8,

              "shape/rectzoom/line/style": 'SolidLine',
              "shape/rectzoom/line/color": "#bbbbbb",
              "shape/rectzoom/line/width": 2,
              # not used -- start
              "shape/rectzoom/sel_line/style": 'SolidLine',
              "shape/rectzoom/sel_line/color": "green",
              "shape/rectzoom/sel_line/width": 2,
              # not used -- end
              "shape/rectzoom/fill/color": "yellow",
              "shape/rectzoom/fill/style": "SolidPattern",
              "shape/rectzoom/fill/alpha": 0.1,
              # not used -- start
              "shape/rectzoom/symbol/marker": 'NoSymbol',
              "shape/rectzoom/symbol/size": 0,
              "shape/rectzoom/symbol/edgecolor": "black",
              "shape/rectzoom/symbol/facecolor": "yellow",
              "shape/rectzoom/symbol/alpha": 1.,
              "shape/rectzoom/sel_symbol/marker": 'Rect',
              "shape/rectzoom/sel_symbol/size": 5,
              "shape/rectzoom/sel_symbol/edgecolor": "black",
              "shape/rectzoom/sel_symbol/facecolor": "yellow",
              "shape/rectzoom/sel_symbol/alpha": 1.,
              # not used -- end

              "shape/axes/border/line/style": 'SolidLine',
              "shape/axes/border/line/color": "magenta",
              "shape/axes/border/line/width": 1,
              "shape/axes/border/sel_line/style": 'SolidLine',
              "shape/axes/border/sel_line/color": "magenta",
              "shape/axes/border/sel_line/width": 2,
              "shape/axes/border/fill/style": "NoBrush",
              "shape/axes/border/fill/color": "magenta",
              "shape/axes/border/fill/alpha": 0.0,
              "shape/axes/border/sel_fill/style": "SolidPattern",
              "shape/axes/border/sel_fill/color": "magenta",
              "shape/axes/border/sel_fill/alpha": 0.3,
              "shape/axes/border/symbol/marker": 'NoSymbol',
              "shape/axes/border/symbol/size": 0,
              "shape/axes/border/symbol/edgecolor": "black",
              "shape/axes/border/symbol/facecolor": "yellow",
              "shape/axes/border/symbol/alpha": 1.,
              "shape/axes/border/sel_symbol/marker": 'NoSymbol',
              "shape/axes/border/sel_symbol/size": 5,
              "shape/axes/border/sel_symbol/edgecolor": "black",
              "shape/axes/border/sel_symbol/facecolor": "yellow",
              "shape/axes/border/sel_symbol/alpha": 1.,
              "shape/axes/arrow_size": 8,
              "shape/axes/arrow_angle": 30,
              "shape/axes/xarrow_pen/style": 'SolidLine',
              "shape/axes/xarrow_pen/color": "red",
              "shape/axes/xarrow_pen/width": 1,
              "shape/axes/xarrow_brush/color": "red",
              "shape/axes/xarrow_brush/alpha": 0.2,
              "shape/axes/yarrow_pen/style": 'SolidLine',
              "shape/axes/yarrow_pen/color": "green",
              "shape/axes/yarrow_pen/width": 1,
              "shape/axes/yarrow_brush/color": "green",
              "shape/axes/yarrow_brush/alpha": 0.2,

              "shape/image_stats/line/style": 'DashLine',
              "shape/image_stats/line/color": "#ff88dc",
              "shape/image_stats/line/width": 1,
              "shape/image_stats/fill/style": "SolidPattern",
              "shape/image_stats/fill/color": "white",
              "shape/image_stats/fill/alpha": 0.1,              
              "shape/image_stats/symbol/marker": 'Rect',
              "shape/image_stats/symbol/size": 3,
              "shape/image_stats/symbol/edgecolor": "#ff55dc",
              "shape/image_stats/symbol/facecolor": "#ff88dc",
              "shape/image_stats/symbol/alpha": .6,
              "shape/image_stats/sel_line/style": 'DashLine',
              "shape/image_stats/sel_line/color": "#ff00dc",
              "shape/image_stats/sel_line/width": 1,
              "shape/image_stats/sel_fill/style": "SolidPattern",
              "shape/image_stats/sel_fill/color": "white",
              "shape/image_stats/sel_fill/alpha": 0.1,              
              "shape/image_stats/sel_symbol/marker": 'Rect',
              "shape/image_stats/sel_symbol/size": 5,
              "shape/image_stats/sel_symbol/edgecolor": "#ff00dc",
              "shape/image_stats/sel_symbol/facecolor": "#ff88dc",
              "shape/image_stats/sel_symbol/alpha": .7,

              "shape/cross_section/line/style": 'DotLine',
              "shape/cross_section/line/color": "#ff5555",
              "shape/cross_section/line/width": 1,
              "shape/cross_section/fill/style": "SolidPattern",
              "shape/cross_section/fill/color": "white",
              "shape/cross_section/fill/alpha": 0.1,              
              "shape/cross_section/symbol/marker": 'Cross',
              "shape/cross_section/symbol/size": 100,
              "shape/cross_section/symbol/edgecolor": "cyan",
              "shape/cross_section/symbol/facecolor": "cyan",
              "shape/cross_section/symbol/alpha": .6,
              "shape/cross_section/sel_line/style": 'DotLine',
              "shape/cross_section/sel_line/color": "#ff0000",
              "shape/cross_section/sel_line/width": 1,
              "shape/cross_section/sel_fill/style": "SolidPattern",
              "shape/cross_section/sel_fill/color": "white",
              "shape/cross_section/sel_fill/alpha": 0.1,              
              "shape/cross_section/sel_symbol/marker": 'Cross',
              "shape/cross_section/sel_symbol/size": 10000,
              "shape/cross_section/sel_symbol/edgecolor": "cyan",
              "shape/cross_section/sel_symbol/facecolor": "cyan",
              "shape/cross_section/sel_symbol/alpha": .7,

              "shape/average_cross_section/line/style": 'DotLine',
              "shape/average_cross_section/line/color": "#ff5555",
              "shape/average_cross_section/line/width": 1,
              "shape/average_cross_section/fill/style": "SolidPattern",
              "shape/average_cross_section/fill/color": "white",
              "shape/average_cross_section/fill/alpha": 0.1,              
              "shape/average_cross_section/symbol/marker": 'Diamond',
              "shape/average_cross_section/symbol/size": 7,
              "shape/average_cross_section/symbol/edgecolor": "#ff5555",
              "shape/average_cross_section/symbol/facecolor": "#ff5555",
              "shape/average_cross_section/symbol/alpha": .6,
              "shape/average_cross_section/sel_line/style": 'DotLine',
              "shape/average_cross_section/sel_line/color": "#ff0000",
              "shape/average_cross_section/sel_line/width": 1,
              "shape/average_cross_section/sel_fill/style": "SolidPattern",
              "shape/average_cross_section/sel_fill/color": "white",
              "shape/average_cross_section/sel_fill/alpha": 0.1,              
              "shape/average_cross_section/sel_symbol/marker": 'Diamond',
              "shape/average_cross_section/sel_symbol/size": 9,
              "shape/average_cross_section/sel_symbol/edgecolor": "#aa0000",
              "shape/average_cross_section/sel_symbol/facecolor": "#ff0000",
              "shape/average_cross_section/sel_symbol/alpha": .7,

              "shape/mask/line/style": 'DotLine',
              "shape/mask/line/color": "#5555ff",
              "shape/mask/line/width": 1,
              "shape/mask/fill/style": "SolidPattern",
              "shape/mask/fill/color": "white",
              "shape/mask/fill/alpha": 0.1,              
              "shape/mask/symbol/marker": 'Rect',
              "shape/mask/symbol/size": 7,
              "shape/mask/symbol/edgecolor": "#5555ff",
              "shape/mask/symbol/facecolor": "#5555ff",
              "shape/mask/symbol/alpha": .6,
              "shape/mask/sel_line/style": 'DotLine',
              "shape/mask/sel_line/color": "#0000ff",
              "shape/mask/sel_line/width": 1,
              "shape/mask/sel_fill/style": "SolidPattern",
              "shape/mask/sel_fill/color": "white",
              "shape/mask/sel_fill/alpha": 0.1,              
              "shape/mask/sel_symbol/marker": 'Rect',
              "shape/mask/sel_symbol/size": 9,
              "shape/mask/sel_symbol/edgecolor": "#0000aa",
              "shape/mask/sel_symbol/facecolor": "#0000ff",
              "shape/mask/sel_symbol/alpha": .7,

              "shape/point/line/style": 'SolidLine',
              "shape/point/line/color": "#ffff00",
              "shape/point/line/width": 1,
              "shape/point/sel_line/style": 'SolidLine',
              "shape/point/sel_line/color": "#00ff00",
              "shape/point/sel_line/width": 1,
              "shape/point/fill/style": "NoBrush",
              "shape/point/sel_fill/style": "NoBrush",
              "shape/point/symbol/marker": 'XCross',
              "shape/point/symbol/size": 9,
              "shape/point/symbol/edgecolor": "#ffff00",
              "shape/point/symbol/facecolor": "#ffff00",
              "shape/point/symbol/alpha": 1.,
              "shape/point/sel_symbol/marker": 'XCross',
              "shape/point/sel_symbol/size": 12,
              "shape/point/sel_symbol/edgecolor": "#00aa00",
              "shape/point/sel_symbol/facecolor": "#00ff00",
              "shape/point/sel_symbol/alpha": .7,

              "shape/segment/line/style": 'SolidLine',
              "shape/segment/line/color": "#ffff00",
              "shape/segment/line/width": 1,
              "shape/segment/sel_line/style": 'SolidLine',
              "shape/segment/sel_line/color": "#00ff00",
              "shape/segment/sel_line/width": 1,
              "shape/segment/fill/style": "NoBrush",
              "shape/segment/sel_fill/style": "NoBrush",
              "shape/segment/symbol/marker": 'XCross',
              "shape/segment/symbol/size": 9,
              "shape/segment/symbol/edgecolor": "#ffff00",
              "shape/segment/symbol/facecolor": "#ffff00",
              "shape/segment/symbol/alpha": 1.,
              "shape/segment/sel_symbol/marker": 'XCross',
              "shape/segment/sel_symbol/size": 12,
              "shape/segment/sel_symbol/edgecolor": "#00aa00",
              "shape/segment/sel_symbol/facecolor": "#00ff00",
              "shape/segment/sel_symbol/alpha": .7,

              "shape/label/symbol/marker": "NoSymbol",
              "shape/label/symbol/size": 0,
              "shape/label/symbol/edgecolor": "white",
              "shape/label/symbol/facecolor": "white",
              "shape/label/border/style": "SolidLine",
              "shape/label/border/color": "#cbcbcb",
              "shape/label/border/width": 0,
              "shape/label/font/size": 8,
              "shape/label/font/family": "default",
              "shape/label/font/bold": False,
              "shape/label/font/italic": False,
              "shape/label/color": "white",
              "shape/label/bgcolor": "black",
              "shape/label/bgalpha": 0.25,
              "shape/label/abspos": False,
              "shape/label/move_anchor": True,
              
              "label/symbol/marker": "NoSymbol",
              "label/symbol/size": 0,
              "label/symbol/edgecolor": "white",
              "label/symbol/facecolor": "white",
              "label/border/style": "SolidLine",
              "label/border/color": "#cbcbcb",
              "label/border/width": 1,
              "label/font/size": 9,
              "label/font/family": "default",
              "label/font/bold": False,
              "label/font/italic": False,
              "label/color": "black",
              "label/bgcolor": "white",
              "label/bgalpha": 0.8,
              "label/anchor": "TL",
              "label/xc": 0,
              "label/yc": 0,
              "label/abspos": False,
              "label/absg": "TR",
              "label/xg": 0.0,
              "label/yg": 0.0,

              # info_label: used in builder.make.computation for example
              "info_label/symbol/marker": "NoSymbol",
              "info_label/symbol/size": 0,
              "info_label/symbol/edgecolor": "white",
              "info_label/symbol/facecolor": "white",
              "info_label/border/style": "SolidLine",
              "info_label/border/color": "#cbcbcb",
              "info_label/border/width": 1,
              "info_label/font/size": 9,
              "info_label/font/family": "default",
              "info_label/font/bold": False,
              "info_label/font/italic": False,
              "info_label/color": "black",
              "info_label/bgcolor": "white",
              "info_label/bgalpha": 0.8,
              "info_label/anchor": "TL",
              "info_label/xc": 0,
              "info_label/yc": 0,
              "info_label/abspos": True,
              "info_label/absg": "TR",
              "info_label/xg": 0.0,
              "info_label/yg": 0.0,

              "legend/border/style": "SolidLine",
              "legend/border/color": "#cbcbcb",
              "legend/border/width": 1,
              "legend/font/size": 8,
              "legend/font/family": "default",
              "legend/font/bold": False,
              "legend/font/italic": False,
              "legend/color": "black",
              "legend/bgcolor": "white",
              "legend/bgalpha": 0.8,
              "legend/anchor": "TR",
              "legend/xc": 0,
              "legend/yc": 0,
              "legend/abspos": True,
              "legend/absg": "TR",
              "legend/xg": 0.0,
              "legend/yg": 0.0,
              },
              
            'histogram':
             {
              "antialiasing": False,
              
              "title/font/size": 11,
              "title/font/bold": False,
              "label/font/size": 9,
              "label/font/bold": False,

              "curve/line/style": 'NoPen',
              "curve/line/color": "green",
              "curve/line/width": 1,
              "curve/symbol/marker": "NoSymbol",
              "curve/symbol/size": 0,
              "curve/symbol/edgecolor": "white",
              "curve/symbol/facecolor": "black",
              "curve/symbol/alpha": 1.,
              "curve/shade": .85,
              "curve/curvestyle": "Steps",
              "curve/label": "",

              "range/line/style": 'SolidLine',
              "range/line/color": "#ff9393",
              "range/line/width": 1,
              "range/sel_line/style": 'SolidLine',
              "range/sel_line/color": "red",
              "range/sel_line/width": 2,
              "range/fill": "red",
              "range/shade": 0.15,
              "range/symbol/marker": "Ellipse",
              "range/symbol/size": 7,
              "range/symbol/edgecolor": "white",
              "range/symbol/facecolor": "#ff9393",
              "range/symbol/alpha": 1.,
              "range/sel_symbol/marker": "Ellipse",
              "range/sel_symbol/size": 9,
              "range/sel_symbol/edgecolor": "white",
              "range/sel_symbol/facecolor": "red",
              "range/sel_symbol/alpha": .9,
              "range/multi/color": "#806060",

              # Default parameters for plot axes
              "axis/title": "",
              "axis/color": "black",
              "axis/title_font/size": 7,
              "axis/title_font/family": "default",
              "axis/title_font/bold": False,
              "axis/title_font/italic": False,
              "axis/ticks_font/size": 7,
              "axis/ticks_font/family": "default",
              "axis/ticks_font/bold": False,
              "axis/ticks_font/italic": False,

              "grid/background": "#FFFFFF",
              "grid/maj_xenabled": True,
              "grid/maj_yenabled": True,
              "grid/maj_line/color": "darkgray",
              "grid/maj_line/width": 1,
              "grid/maj_line/style": 'DotLine',
              "grid/min_xenabled": True,
              "grid/min_yenabled": False,
              "grid/min_line/color": "#eaeaea",
              "grid/min_line/width": 1,
              "grid/min_line/style": 'DotLine',

              "marker/curve/symbol/marker": 'Ellipse',
              "marker/curve/symbol/edgecolor": "black",
              "marker/curve/symbol/facecolor": "blue",
              "marker/curve/symbol/alpha": .8,
              "marker/curve/symbol/size": 8,
              "marker/curve/text/font/size": 8,
              "marker/curve/text/font/family": "default",
              "marker/curve/text/font/bold": False,
              "marker/curve/text/font/italic": False,
              "marker/curve/text/textcolor": "darkBlue",
              "marker/curve/text/background_color": "blue",
              "marker/curve/text/background_alpha": 0.2,
              "marker/curve/pen/style": "SolidLine",
              "marker/curve/pen/color": "black",
              "marker/curve/pen/width": 0,
              "marker/curve/markerstyle": "NoLine",

              "marker/cross/symbol/marker": 'Cross',
              "marker/cross/symbol/edgecolor": "black",
              "marker/cross/symbol/facecolor": "red",
              "marker/cross/symbol/alpha": 1.,
              "marker/cross/symbol/size": 8,
              "marker/cross/text/font/family": "default",
              "marker/cross/text/font/size": 8,
              "marker/cross/text/font/bold": False,
              "marker/cross/text/font/italic": False,
              "marker/cross/text/textcolor": "darkBlue",
              "marker/cross/text/background_color": "blue",
              "marker/cross/text/background_alpha": 0.2,
              "marker/cross/pen/style": "SolidLine",
              "marker/cross/pen/color": "black",
              "marker/cross/pen/width": 1,
              "marker/cross/markerstyle": "Cross",
              },
              
            'cross_section':
             {
              "antialiasing": False,
              
              "title/font/size": 11,
              "title/font/bold": False,
              "label/font/size": 9,
              "label/font/bold": False,

              "curve/line/style": 'SolidLine',
              "curve/line/color": "blue",
              "curve/line/width": 1,
              "curve/symbol/marker": "NoSymbol",
              "curve/symbol/size": 0,
              "curve/symbol/edgecolor": "white",
              "curve/symbol/facecolor": "black",
              "curve/symbol/alpha": 1.,
              "curve/shade": 0.,
              "curve/label": "",

              "range/line/style": 'SolidLine',
              "range/line/color": "#ff9393",
              "range/line/width": 1,
              "range/sel_line/style": 'SolidLine',
              "range/sel_line/color": "red",
              "range/sel_line/width": 2,
              "range/fill": "red",
              "range/shade": 0.15,
              "range/symbol/marker": "NoSymbol",
              "range/symbol/size": 5,
              "range/symbol/edgecolor": "black",
              "range/symbol/facecolor": "yellow",
              "range/symbol/alpha": 1.,
              "range/sel_symbol/marker": "Ellipse",
              "range/sel_symbol/size": 9,
              "range/sel_symbol/edgecolor": "white",
              "range/sel_symbol/facecolor": "red",
              "range/sel_symbol/alpha": .9,
              "range/multi/color": "#806060",

              # Default parameters for plot axes
              "axis/title": "",
              "axis/color": "black",
              "axis/title_font/size": 7,
              "axis/title_font/family": "default",
              "axis/title_font/bold": False,
              "axis/title_font/italic": False,
              "axis/ticks_font/size": 7,
              "axis/ticks_font/family": "default",
              "axis/ticks_font/bold": False,
              "axis/ticks_font/italic": False,

              "grid/background": "#FFFFFF",
              "grid/maj_xenabled": True,
              "grid/maj_yenabled": True,
              "grid/maj_line/color": "darkgray",
              "grid/maj_line/width": 1,
              "grid/maj_line/style": 'DotLine',
              "grid/min_xenabled": False,
              "grid/min_yenabled": False,
              "grid/min_line/color": "#eaeaea",
              "grid/min_line/width": 1,
              "grid/min_line/style": 'DotLine',

              "marker/curve/symbol/marker": 'Ellipse',
              "marker/curve/symbol/edgecolor": "black",
              "marker/curve/symbol/facecolor": "blue",
              "marker/curve/symbol/alpha": .8,
              "marker/curve/symbol/size": 8,
              "marker/curve/text/font/size": 8,
              "marker/curve/text/font/family": "default",
              "marker/curve/text/font/bold": False,
              "marker/curve/text/font/italic": False,
              "marker/curve/text/textcolor": "darkBlue",
              "marker/curve/text/background_color": "blue",
              "marker/curve/text/background_alpha": 0.2,
              "marker/curve/pen/style": "SolidLine",
              "marker/curve/pen/color": "black",
              "marker/curve/pen/width": 0,
              "marker/curve/markerstyle": "NoLine",

              "marker/cross/symbol/marker": 'Cross',
              "marker/cross/symbol/edgecolor": "black",
              "marker/cross/symbol/facecolor": "red",
              "marker/cross/symbol/alpha": 1.,
              "marker/cross/symbol/size": 8,
              "marker/cross/text/font/family": "default",
              "marker/cross/text/font/size": 8,
              "marker/cross/text/font/bold": False,
              "marker/cross/text/font/italic": False,
              "marker/cross/text/textcolor": "darkBlue",
              "marker/cross/text/background_color": "blue",
              "marker/cross/text/background_alpha": 0.2,
              "marker/cross/pen/style": "SolidLine",
              "marker/cross/pen/color": "black",
              "marker/cross/pen/width": 1,
              "marker/cross/markerstyle": "Cross",
              },
              
            }

from guidata.config import CONF
CONF.update_defaults(DEFAULTS)



# -*- coding: utf-8 -*-
#
# Copyright © 2009-2010 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""
guiqwt.qtdesigner
-----------------

The `qtdesigner` module provides QtDesigner helper functions for `guiqwt`:
    * :py:func:`guiqwt.qtdesigner.loadui`
    * :py:func:`guiqwt.qtdesigner.compileui`
    * :py:func:`guiqwt.qtdesigner.create_qtdesigner_plugins`
    
Reference
~~~~~~~~~

.. autofunction:: loadui
.. autofunction:: compileui
.. autofunction:: create_qtdesigner_plugin
"""

from qtpy import uic
from qtpy.QtDesigner import QPyDesignerCustomWidgetPlugin
from qtpy.QtGui import QIcon

from guidata.configtools import get_icon
from qtpy.py3compat import io


def loadui(fname, replace_class="QwtPlot"):
    """
    Return Widget or Window class from QtDesigner ui file 'fname'
    
    The loadUiType function (PyQt4.uic) doesn't work correctly with guiqwt
    QtDesigner plugins because they don't inheritate from a PyQt4.QtGui
    object.
    """
    uifile_text = open(fname).read().replace(replace_class, "QFrame")
    ui, base_class = uic.loadUiType(io.StringIO(uifile_text))

    class Form(base_class, ui):
        def __init__(self, parent=None):
            super(Form, self).__init__(parent)
            self.setupUi(self)

    return Form


def compileui(fname, replace_class="QwtPlot"):
    uifile_text = open(fname).read().replace("QwtPlot", "QFrame")
    uic.compileUi(
        io.StringIO(uifile_text),
        open(fname.replace(".ui", "_ui.py"), "w"),
        pyqt3_wrapper=True,
    )


def create_qtdesigner_plugin(
    group,
    module_name,
    class_name,
    widget_options={},
    icon=None,
    tooltip="",
    whatsthis="",
):
    """Return a custom QtDesigner plugin class
    
    Example:
    create_qtdesigner_plugin(group = "guiqwt", module_name = "guiqwt.image",
                             class_name = "ImageWidget", icon = "image.png",
                             tooltip = "", whatsthis = ""):
    """
    Widget = getattr(__import__(module_name, fromlist=[class_name]), class_name)

    class CustomWidgetPlugin(QPyDesignerCustomWidgetPlugin):
        def __init__(self, parent=None):
            QPyDesignerCustomWidgetPlugin.__init__(self)
            self.initialized = False

        def initialize(self, core):
            if self.initialized:
                return
            self.initialized = True

        def isInitialized(self):
            return self.initialized

        def createWidget(self, parent):
            return Widget(parent, **widget_options)

        def name(self):
            return class_name

        def group(self):
            return group

        def icon(self):
            if icon is not None:
                return get_icon(icon)
            else:
                return QIcon()

        def toolTip(self):
            return tooltip

        def whatsThis(self):
            return whatsthis

        def isContainer(self):
            return False

        def domXml(self):
            return '<widget class="%s" name="%s" />\n' % (
                class_name,
                class_name.lower(),
            )

        def includeFile(self):
            return module_name

    return CustomWidgetPlugin



# -*- coding: utf-8 -*-
#
# Copyright © 2010 CEA
# Ludovic Aubry
# Licensed under the terms of the CECILL License
# (see guidata/__init__.py for details)

"""CurvePlotDialog test"""


# ===============================================================================
# TODO: Make this test work!!
# ===============================================================================


SHOW = False  # Show test in GUI-based test launcher

from qtpy.QtGui import QFont

from guiqwt.baseplot import BasePlot
from guiqwt.plot import CurveDialog, CurveWidget, PlotManager
from guiqwt.builder import make
from guiqwt.curve import CurvePlot


class MyPlotDialog(CurveDialog):
    def create_plot(self, options):
        manager = PlotManager(None)
        self.plotwidget = CurveWidget(self, manager=manager, **options)
        manager.set_main(self.plotwidget)
        plot1 = CurvePlot(title="TL")
        plot2 = CurvePlot(title="TR")
        plot3 = CurvePlot(title="BL")
        plot4 = CurvePlot(title="BR")
        self.plotwidget.add_plot(plot1, 0, 0, "1")
        self.plotwidget.add_plot(plot2, 0, 1, "2")
        self.plotwidget.add_plot(plot3, 1, 0, "3")
        self.plotwidget.add_plot(plot4, 1, 1, "4")
        self.plotwidget.finalize()
        manager.synchronize_axis(BasePlot.X_BOTTOM, ["1", "3"])
        manager.synchronize_axis(BasePlot.X_BOTTOM, ["2", "4"])
        manager.synchronize_axis(BasePlot.Y_LEFT, ["1", "2"])
        manager.synchronize_axis(BasePlot.Y_LEFT, ["3", "4"])

        self.layout.addWidget(self.plotwidget, 0, 0)


def plot(items1, items2, items3, items4):
    win = MyPlotDialog(
        edit=False,
        toolbar=True,
        wintitle="CurvePlotDialog test",
        options=dict(title="Title", xlabel="xlabel", ylabel="ylabel"),
    )
    items = [items1, items2, items3, items4]
    for i, plot in enumerate(win.plotwidget.plots):
        for item in items[i]:
            plot.add_item(item)
        plot.set_axis_font("left", QFont("Courier"))
        plot.set_items_readonly(False)
    win.get_panel("itemlist").show()
    win.show()
    win.exec_()


def test():
    """Test"""
    # -- Create QApplication
    import guidata

    _app = guidata.qapplication()
    # --
    from numpy import linspace, sin

    x = linspace(-10, 10, 200)
    dy = x / 100.0
    y = sin(sin(sin(x)))
    x2 = linspace(-10, 10, 20)
    y2 = sin(sin(sin(x2)))
    plot(
        [
            make.curve(x, y, color="b"),
            make.label(
                "Relative position <b>outside</b>", (x[0], y[0]), (-10, -10), "BR"
            ),
        ],
        [make.curve(x2, y2, color="g"),],
        [
            make.curve(x, sin(2 * y), color="r"),
            make.label("Relative position <i>inside</i>", (x[0], y[0]), (10, 10), "TL"),
        ],
        [
            make.merror(x, y / 2, dy),
            make.label("Absolute position", "R", (0, 0), "R"),
            make.legend("TR"),
        ],
    )


if __name__ == "__main__":
    test()



# -*- coding: utf-8 -*-
#
# Copyright © 2009-2010 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""Image with custom X/Y axes linear scales"""

from __future__ import print_function

SHOW = True # Show test in GUI-based test launcher

from guiqwt.plot import ImageDialog
from guiqwt.builder import make

import numpy as np

def imshow( x, y, data ):
    win = ImageDialog(edit=False, toolbar=True,
                      wintitle="Image with custom X/Y axes scales",
                      options=dict(xlabel="x (a.u.)", ylabel="y (a.u.)",
                                   yreverse=False))
    item = make.xyimage(x, y, data)
    plot = win.get_plot()
    plot.add_item(item)
    win.show()
    win.exec_()

def compute_image():
    N = 2000
    T = np.float32
    x = np.array(np.linspace(-5, 5, N), T)
    img = np.zeros( (N, N), T )
    x.shape = (1, N)
    img += x**2
    x.shape = (N, 1)
    img += x**2
    img = np.cos(img)
    x.shape = (N,)
    for k in range( -5, 5 ):
        i = x.searchsorted(k)
        if k < 0 :
            v = -1.1
        else:
            v = 1.1
        img[i,:] = v
        img[:, i] = v
    m1, m2, m3, m4 = -1.1, -0.3, 0.3, 1.1
    K = 100
    img[:K, :K] = m1     # (0,0)
    img[:K, -K:] = m2    # (0,N)
    img[-K:, -K:] = m3   # (N,N)
    img[-K:, :K] = m4    # (N,0)
    #img = array( 30000*(img+1.1), uint16 )
    img = img + np.random.normal(0., 0.05, size=(N, N))
    print(img.dtype)
    return x, (x+5)**0.6, img

def test():
    """Test"""
    # -- Create QApplication
    import guidata
    _app = guidata.qapplication()
    # --    
    imshow(*compute_image())

if __name__ == "__main__":
    test()
